#include <stdlib.h>
#include <string.h>
#include "config.h"
#include "ssl.h"
#include "conc.h"
#include "groups.h"
#include "terminal.h"
#include "log.h"
#include "error.h"
#include "subscribed.h"
#include "configuration.h"
#include "client-message.h"

typedef struct _ChangePasswordDialog {
    Conc *conc;
    GladeXML *xml;
} ChangePasswordDialog;

struct subscribed_data {
    Conc *conc;
    gchar *cgroup;
};

/* 
 * Glade widget names:
 */
#define MAIN_WINDOW			"main window"
#define MAIN_WINDOW_NOTEBOOK		"mwnotebook"
#define LOGIN_DIALOG			"login dialog"
#define LOGIN_DIALOG_HOST		"ldhost"
#define LOGIN_DIALOG_USER		"lduser"
#define LOGIN_DIALOG_PASSWORD		"ldpassword"
#define LOGIN_DIALOG_LOGIN		"ldlogin"
#define CHANGE_PASSWORD_DIALOG		"change password dialog"
#define CHANGE_PASSWORD_DIALOG_OLD	"cpold"
#define CHANGE_PASSWORD_DIALOG_NEW1	"cpnew1"
#define CHANGE_PASSWORD_DIALOG_NEW2	"cpnew2"
#define CHANGE_PASSWORD_DIALOG_OK	"cpok"

/*
 * Glade handler names:
 */
#define MENU_CONC_CHANGE_PASSWORD	"change password"
#define MENU_EDIT_SUBSCRIBED		"edit subscribed"
#define MENU_EDIT_CONFIGURATION		"edit configuration"
#define MENU_VIEW_CONSOLE		"view console"
#define MENU_HELP_CONTENTS		"help"
#define MENU_HELP_ABOUT			"about"
#define MENU_VIEW_LOG			"view log"
#define MAIN_PAGE_SWITCH		"switch page"
#define LOGIN_DIALOG_REPONSE		"response"
#define LOGIN_DIALOG_CHANGED		"changed"
#define CHANGE_PASSWORD_DIALOG_CHANGED	"changed"
#define CHANGE_PASSWORD_DIALOG_RESPONSE	"response"

static void
change_password_done(MessageQueue q,
	const MessageArg *arg,
	gpointer data) {
    Conc *conc = data;
    GError *e = message_arg_error_check(arg);

    if (e != NULL) {
	warning_dialog(conc, _("Failed to change password"),
		_("Could not change password (%s)"), e->message);
	g_clear_error(&e);
    } else {
	info_dialog(conc, _("Password changed"),
		_("Your password was changed successfully"));
    }
}

static void
change_password_changed(GtkEditable *editable,
	gpointer user_data) {
    ChangePasswordDialog *cpd = user_data;
    const gchar *new1 = NULL;
    const gchar *new2 = NULL;

    new1 = gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(cpd->xml,
		    CHANGE_PASSWORD_DIALOG_NEW1)));
    new2 = gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(cpd->xml,
		    CHANGE_PASSWORD_DIALOG_NEW2)));
    if (new1 != NULL && strlen(new1) > 5 &&
	    new2 != NULL && strlen(new2) > 5 &&
	    strcmp(new1, new2) == 0) {
	g_object_set(glade_xml_get_widget(cpd->xml, CHANGE_PASSWORD_DIALOG_OK),
		"sensitive", TRUE, NULL);
    } else {
	g_object_set(glade_xml_get_widget(cpd->xml, CHANGE_PASSWORD_DIALOG_OK),
		"sensitive", FALSE, NULL);
    }
}

static void
change_password_response(GtkDialog *dialog,
	gint arg1,
	gpointer user_data) {
    ChangePasswordDialog *cpd = user_data;

    if (arg1 == GTK_RESPONSE_OK) {
	MessageArg *a = message_arg_list("", NULL);
	GError *e = NULL;

	a->list = g_list_prepend(a->list, message_arg_string("old",
		    gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(cpd->xml,
		    CHANGE_PASSWORD_DIALOG_OLD)))));
	a->list = g_list_prepend(a->list, message_arg_string("new",
		    gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(cpd->xml,
		    CHANGE_PASSWORD_DIALOG_NEW1)))));
	if (message_send(cpd->conc->msg_q, CLIENT_MESSAGE_CHANGE_PASSWORD,
		    a, change_password_done, cpd->conc, &e) == FALSE) {
	    error_dialog(cpd->conc, _("Internal error"),
		    _("Failed to send message (%s)"), e->message);
	    g_clear_error(&e);
	}
	message_arg_free(a);
    }
    gtk_widget_destroy(glade_xml_get_widget(cpd->xml, CHANGE_PASSWORD_DIALOG));
    g_free(cpd);
}

static void
menu_conc_change_password(GtkMenuItem *menuitem,
	gpointer user_data) {
    Conc *conc = user_data;
    ChangePasswordDialog *cpd = g_new0(struct _ChangePasswordDialog, 1);

    cpd->conc = conc;
    cpd->xml = glade_xml_new(conc->glade_file, CHANGE_PASSWORD_DIALOG, NULL);
    glade_xml_signal_connect_data(cpd->xml, CHANGE_PASSWORD_DIALOG_RESPONSE,
	    (GCallback)change_password_response, cpd);
    glade_xml_signal_connect_data(cpd->xml, CHANGE_PASSWORD_DIALOG_CHANGED,
	    (GCallback)change_password_changed, cpd);
}

extern gchar *
get_combobox_text(GladeXML *xml,
	const gchar *widget,
	GtkTreeModel *model,
	gint column) {
    GtkTreeIter iter;
    GtkComboBox *cb;
    gchar *ret;

    cb = GTK_COMBO_BOX(glade_xml_get_widget(xml, widget));
    if (model == NULL) {
	model = gtk_combo_box_get_model(cb);
	column = 0;
    }
    if (gtk_combo_box_get_active_iter(cb, &iter) == FALSE &&
	gtk_tree_model_get_iter_first(model, &iter) == FALSE) {
	return NULL;
    }
    gtk_tree_model_get(model, &iter, column, &ret, -1);
    return ret;
}

extern void
set_combobox_text(GladeXML *xml,
	const gchar *widget,
	GtkTreeModel *model,
	gint column,
	const gchar *value) {
    GtkTreeIter iter;
    GtkComboBox *cb;
    gchar *val;

    cb = GTK_COMBO_BOX(glade_xml_get_widget(xml, widget));
    if (model == NULL) {
	model = gtk_combo_box_get_model(cb);
	column = 0;
    }
    if (gtk_tree_model_get_iter_first(model, &iter)) {
	do {
	    gtk_tree_model_get(model, &iter, column, &val, -1);
	    if (!strcmp(val, value)) {
		g_free(val);
		gtk_combo_box_set_active_iter(cb, &iter);
		break;
	    }
	    g_free(val);
	} while (gtk_tree_model_iter_next(model, &iter));
    }
}

static void
menu_edit_subscribed(GtkMenuItem *menuitem,
	gpointer user_data) {
    Conc *conc = user_data;

    edit_subscribed_show(conc);
}

static void
menu_edit_configuration(GtkMenuItem *menuitem,
	gpointer user_data) {
    Conc *conc = user_data;

    configuration_show(conc);
}

static void
menu_view_console(GtkMenuItem *menuitem,
	gpointer user_data) {
    Conc *conc = user_data;

    g_assert(conc->current_group);
    g_assert(conc->current_console);
    terminal_open(conc, conc->current_group, conc->current_console);
}

static void
menu_view_log(GtkMenuItem *menuitem,
	gpointer user_data) {
    Conc *conc = user_data;

    log_view(conc, NULL);
}

static void
menu_help_contents(GtkMenuItem *menuitem,
	gpointer user_data) {
    Conc *conc = user_data;
    GError *e = NULL;

    if (gnome_help_display("conc", NULL, &e) == FALSE) {
	warning_dialog(conc, "Could not display help", "%s", e->message);
    }
}

static void
menu_help_about(GtkMenuItem *menuitem,
	gpointer user_data) {
    Conc *conc = user_data;
    gchar *authors[] = { "James Carter <james@jfc.org.uk>", NULL };

    gtk_show_about_dialog(GTK_WINDOW(glade_xml_get_widget(conc->main_window,
	    MAIN_WINDOW)),
	    "version", VERSION,
	    "website", "http://www.jfc.org.uk/software/conc.html",
	    "authors", authors,
	    "comments", _("Serial console concentrator."),
	    "license", _("This program is distributed under the terms of the\n"
		         "GNU General Public license as published by the Free\n"
			 "Software Foundation; either version 2 of the License,\n"
			 "or (at your option) any later version. A copy of\n"
			 "this license can be found in the file COPYING\n"
			 "included with the source code of this program."),
	    "copyright", _("Copyright 2002-2009 James Carter"),
	    "logo", gdk_pixbuf_new_from_file(CONCDATADIR "/conc.png", NULL),
	    NULL);
}

static void
about_url(GtkAboutDialog *about,
	const gchar *link,
	gpointer data) {
    gnome_url_show(link, NULL);
}

static void
about_email(GtkAboutDialog *about,
	const gchar *link,
	gpointer data) {
    gchar *url;

    url = g_strdup_printf("mailto:%s", link);
    gnome_url_show(url, NULL);
    g_free(url);
}

static void
cgroup_change(MessageQueue q,
	const gchar *name,
	MessageID id,
	const MessageArg *arg,
	gpointer data) {
    Conc *conc = data;
    MessageArg *cgroup;
    MessageArg *console;
    MessageArg *status;
    MessageArg *a;

    if (message_arg_list_explode(arg,
		"cgroup", MESSAGE_ARG_STRING, &cgroup,
		"console", MESSAGE_ARG_STRING, &console,
		"status", MESSAGE_ARG_UINT8, &status,
		NULL, 0, NULL) == FALSE) {
	g_printerr("malformed message in cgroup_change report!\n");
	return;
    }
    if (status->uint8 == 0) {
	group_remove_console(conc, cgroup->string->str, console->string->str);
    } else {
	group_add_console(conc, cgroup->string->str, console->string->str);
    }
    a = message_arg_none("");
    message_respond(q, id, a, NULL);
    message_arg_free(a);
}

static void
cgroup_add(MessageQueue q,
	const gchar *name,
	MessageID id,
	const MessageArg *arg,
	gpointer data) {
    Conc *conc = data;
    MessageArg *a;
    GtkTreeIter iter;

    if (arg->type != MESSAGE_ARG_STRING) {
	g_printerr("malformed message in cgroup_add report!\n");
	return;
    }
    gtk_list_store_append(conc->all_groups, &iter);
    gtk_list_store_set(conc->all_groups, &iter,
	    GROUPS_COLUMN_NAME, arg->string->str,
	    -1);
    a = message_arg_none("");
    message_respond(q, id, a, NULL);
    message_arg_free(a);
}

static void
cgroup_remove(MessageQueue q,
	const gchar *name,
	MessageID id,
	const MessageArg *arg,
	gpointer data) {
    Conc *conc = data;
    MessageArg *a;
    GtkTreeIter iter;

    if (arg->type != MESSAGE_ARG_STRING) {
	g_printerr("malformed message in cgroup_remove report!\n");
	return;
    }
    group_delete(conc, arg->string->str);
    groups_save(conc);
    if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(conc->all_groups),
		&iter)) {
	do {
	    gchar *cgroup;

	    gtk_tree_model_get(GTK_TREE_MODEL(conc->all_groups), &iter,
		    GROUPS_COLUMN_NAME, &cgroup, -1);
	    if (!strcmp(cgroup, arg->string->str)) {
		g_free(cgroup);
		gtk_list_store_remove(conc->all_groups, &iter);
		break;
	    }
	    g_free(cgroup);
	} while (gtk_tree_model_iter_next(GTK_TREE_MODEL(conc->all_groups),
		    &iter));
    }
    a = message_arg_none("");
    message_respond(q, id, a, NULL);
    message_arg_free(a);
}

static void
cgroup_rename(MessageQueue q,
	const gchar *name,
	MessageID id,
	const MessageArg *arg,
	gpointer data) {
    Conc *conc = data;
    MessageArg *a;
    MessageArg *oldname;
    MessageArg *newname;
    GtkTreeIter iter;

    if (arg->type != MESSAGE_ARG_LIST) {
	g_printerr("malformed message in cgroup_rename report!\n");
	return;
    }
    if (message_arg_list_explode(arg,
		"cgroup", MESSAGE_ARG_STRING, &oldname,
		"newname", MESSAGE_ARG_STRING, &newname,
		NULL, 0, NULL) == FALSE) {
	g_printerr("malformed message in cgroup_rename report!\n");
	return;
    }
    if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(conc->all_groups),
		&iter)) {
	do {
	    gchar *cgroup = NULL;

	    gtk_tree_model_get(GTK_TREE_MODEL(conc->all_groups), &iter,
		    USERS_COLUMN_NAME, &cgroup, -1);
	    if (cgroup == NULL) {
		continue;
	    }
	    if (strcmp(oldname->string->str, cgroup) == 0) {
		gtk_list_store_set(conc->all_groups, &iter,
		    USERS_COLUMN_NAME, newname->string->str,
		    -1);
		g_free(cgroup);
		break;
	    }
	    g_free(cgroup);
	} while (gtk_tree_model_iter_next(GTK_TREE_MODEL(conc->all_groups), &iter));
    }
    groups_rename_cgroup(conc, oldname->string->str, newname->string->str);
    terminals_rename_cgroup(conc, oldname->string->str, newname->string->str);
    a = message_arg_none("");
    message_respond(q, id, a, NULL);
    message_arg_free(a);
}

static void
console_add(MessageQueue q,
	const gchar *name,
	MessageID id,
	const MessageArg *arg,
	gpointer data) {
    Conc *conc = data;
    MessageArg *a;
    GtkTreeIter iter;

    if (arg->type != MESSAGE_ARG_STRING) {
	g_printerr("malformed message in console_add report!\n");
	return;
    }
    gtk_list_store_append(conc->all_consoles, &iter);
    gtk_list_store_set(conc->all_consoles, &iter,
	    CONSOLES_COLUMN_NAME, arg->string->str,
	    -1);
    a = message_arg_none("");
    message_respond(q, id, a, NULL);
    message_arg_free(a);
}

static void
console_remove(MessageQueue q,
	const gchar *name,
	MessageID id,
	const MessageArg *arg,
	gpointer data) {
    Conc *conc = data;
    MessageArg *a;
    GtkTreeIter iter;

    if (arg->type != MESSAGE_ARG_STRING) {
	g_printerr("malformed message in console_remove report!\n");
	return;
    }
    if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(conc->all_consoles),
		&iter)) {
	do {
	    gchar *console;

	    gtk_tree_model_get(GTK_TREE_MODEL(conc->all_consoles), &iter,
		    CONSOLES_COLUMN_NAME, &console, -1);
	    if (!strcmp(console, arg->string->str)) {
		g_free(console);
		gtk_list_store_remove(conc->all_consoles, &iter);
		break;
	    }
	    g_free(console);
	} while (gtk_tree_model_iter_next(GTK_TREE_MODEL(conc->all_consoles),
		    &iter));
    }
    a = message_arg_none("");
    message_respond(q, id, a, NULL);
    message_arg_free(a);
}

static void
console_rename(MessageQueue q,
	const gchar *name,
	MessageID id,
	const MessageArg *arg,
	gpointer data) {
    Conc *conc = data;
    MessageArg *a;
    MessageArg *oldname;
    MessageArg *newname;
    GtkTreeIter iter;

    if (arg->type != MESSAGE_ARG_LIST) {
	g_printerr("malformed message in console_rename report!\n");
	return;
    }
    if (message_arg_list_explode(arg,
		"console", MESSAGE_ARG_STRING, &oldname,
		"newname", MESSAGE_ARG_STRING, &newname,
		NULL, 0, NULL) == FALSE) {
	g_printerr("malformed message in console_rename report!\n");
	return;
    }
    if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(conc->all_consoles),
		&iter)) {
	do {
	    gchar *console = NULL;

	    gtk_tree_model_get(GTK_TREE_MODEL(conc->all_consoles), &iter,
		    USERS_COLUMN_NAME, &console, -1);
	    if (console == NULL) {
		continue;
	    }
	    if (strcmp(oldname->string->str, console) == 0) {
		gtk_list_store_set(conc->all_consoles, &iter,
		    USERS_COLUMN_NAME, newname->string->str,
		    -1);
		g_free(console);
		break;
	    }
	    g_free(console);
	} while (gtk_tree_model_iter_next(GTK_TREE_MODEL(conc->all_consoles), &iter));
    }
    groups_rename_console(conc, oldname->string->str, newname->string->str);
    terminals_rename_console(conc, oldname->string->str, newname->string->str);
    a = message_arg_none("");
    message_respond(q, id, a, NULL);
    message_arg_free(a);
}

static void
log_add(MessageQueue q,
	const gchar *name,
	MessageID id,
	const MessageArg *arg,
	gpointer data) {
    Conc *conc = data;
    MessageArg *a;
    GtkTreeIter iter;

    if (arg->type != MESSAGE_ARG_STRING) {
	g_printerr("malformed message in log_add report!\n");
	return;
    }
    gtk_list_store_append(conc->all_logs, &iter);
    gtk_list_store_set(conc->all_logs, &iter,
	    LOGS_COLUMN_NAME, arg->string->str,
	    -1);
    a = message_arg_none("");
    message_respond(q, id, a, NULL);
    message_arg_free(a);
}

static void
log_remove(MessageQueue q,
	const gchar *name,
	MessageID id,
	const MessageArg *arg,
	gpointer data) {
    Conc *conc = data;
    MessageArg *a;
    GtkTreeIter iter;

    if (arg->type != MESSAGE_ARG_STRING) {
	g_printerr("malformed message in log_remove report!\n");
	return;
    }
    g_assert(gtk_tree_model_get_iter_first(GTK_TREE_MODEL(conc->all_logs),
		&iter));
    do {
	gchar *log = NULL;

	gtk_tree_model_get(GTK_TREE_MODEL(conc->all_logs), &iter,
		LOGS_COLUMN_NAME, &log, -1);
	if (log == NULL) {
	    continue;
	}
	if (strcmp(arg->string->str, log) == 0) {
	    gtk_list_store_remove(conc->all_logs, &iter);
	    g_free(log);
	    break;
	}
	g_free(log);
    } while (gtk_tree_model_iter_next(GTK_TREE_MODEL(conc->all_logs), &iter));
    a = message_arg_none("");
    message_respond(q, id, a, NULL);
    message_arg_free(a);
}

static void
log_rename(MessageQueue q,
	const gchar *name,
	MessageID id,
	const MessageArg *arg,
	gpointer data) {
    Conc *conc = data;
    MessageArg *a;
    MessageArg *oldname;
    MessageArg *newname;
    GtkTreeIter iter;

    if (arg->type != MESSAGE_ARG_LIST) {
	g_printerr("malformed message in log_rename report!\n");
	return;
    }
    if (message_arg_list_explode(arg,
		"log", MESSAGE_ARG_STRING, &oldname,
		"newname", MESSAGE_ARG_STRING, &newname,
		NULL, 0, NULL) == FALSE) {
	g_printerr("malformed message in log_rename report!\n");
	return;
    }
    g_assert(gtk_tree_model_get_iter_first(GTK_TREE_MODEL(conc->all_logs),
		&iter));
    do {
	gchar *log = NULL;

	gtk_tree_model_get(GTK_TREE_MODEL(conc->all_logs), &iter,
		LOGS_COLUMN_NAME, &log, -1);
	if (log == NULL) {
	    continue;
	}
	if (strcmp(oldname->string->str, log) == 0) {
	    gtk_list_store_set(conc->all_logs, &iter,
		LOGS_COLUMN_NAME, newname->string->str,
		-1);
	    g_free(log);
	    break;
	}
	g_free(log);
    } while (gtk_tree_model_iter_next(GTK_TREE_MODEL(conc->all_logs), &iter));
    a = message_arg_none("");
    message_respond(q, id, a, NULL);
    message_arg_free(a);
}

static void
user_add(MessageQueue q,
	const gchar *name,
	MessageID id,
	const MessageArg *arg,
	gpointer data) {
    Conc *conc = data;
    MessageArg *a;
    GtkTreeIter iter;

    if (arg->type != MESSAGE_ARG_STRING) {
	g_printerr("malformed message in user_add report!\n");
	return;
    }
    gtk_list_store_append(conc->all_users, &iter);
    gtk_list_store_set(conc->all_users, &iter,
	    USERS_COLUMN_NAME, arg->string->str,
	    -1);
    a = message_arg_none("");
    message_respond(q, id, a, NULL);
    message_arg_free(a);
}

static void
user_remove(MessageQueue q,
	const gchar *name,
	MessageID id,
	const MessageArg *arg,
	gpointer data) {
    Conc *conc = data;
    MessageArg *a;
    GtkTreeIter iter;

    if (arg->type != MESSAGE_ARG_STRING) {
	g_printerr("malformed message in user_remove report!\n");
	return;
    }
    g_assert(gtk_tree_model_get_iter_first(GTK_TREE_MODEL(conc->all_users),
		&iter));
    do {
	gchar *user = NULL;

	gtk_tree_model_get(GTK_TREE_MODEL(conc->all_users), &iter,
		USERS_COLUMN_NAME, &user, -1);
	if (user == NULL) {
	    continue;
	}
	if (strcmp(arg->string->str, user) == 0) {
	    gtk_list_store_remove(conc->all_users, &iter);
	    g_free(user);
	    break;
	}
	g_free(user);
    } while (gtk_tree_model_iter_next(GTK_TREE_MODEL(conc->all_users), &iter));
    a = message_arg_none("");
    message_respond(q, id, a, NULL);
    message_arg_free(a);
}

static void
user_rename(MessageQueue q,
	const gchar *name,
	MessageID id,
	const MessageArg *arg,
	gpointer data) {
    Conc *conc = data;
    MessageArg *a;
    MessageArg *oldname;
    MessageArg *newname;
    GtkTreeIter iter;

    if (arg->type != MESSAGE_ARG_LIST) {
	g_printerr("malformed message in user_rename report!\n");
	return;
    }
    if (message_arg_list_explode(arg,
		"user", MESSAGE_ARG_STRING, &oldname,
		"newname", MESSAGE_ARG_STRING, &newname,
		NULL, 0, NULL) == FALSE) {
	g_printerr("malformed message in user_rename report!\n");
	return;
    }
    g_assert(gtk_tree_model_get_iter_first(GTK_TREE_MODEL(conc->all_users),
		&iter));
    do {
	gchar *user = NULL;

	gtk_tree_model_get(GTK_TREE_MODEL(conc->all_users), &iter,
		USERS_COLUMN_NAME, &user, -1);
	if (user == NULL) {
	    continue;
	}
	if (strcmp(oldname->string->str, user) == 0) {
	    gtk_list_store_set(conc->all_users, &iter,
		USERS_COLUMN_NAME, newname->string->str,
		-1);
	    g_free(user);
	    break;
	}
	g_free(user);
    } while (gtk_tree_model_iter_next(GTK_TREE_MODEL(conc->all_users), &iter));
    a = message_arg_none("");
    message_respond(q, id, a, NULL);
    message_arg_free(a);
}

static void
server_add(MessageQueue q,
	const gchar *name,
	MessageID id,
	const MessageArg *arg,
	gpointer data) {
    Conc *conc = data;
    MessageArg *a;
    GtkTreeIter iter;

    if (arg->type != MESSAGE_ARG_STRING) {
	g_printerr("malformed message in server_add report!\n");
	return;
    }
    gtk_list_store_append(conc->all_servers, &iter);
    gtk_list_store_set(conc->all_servers, &iter,
	    SERVERS_COLUMN_NAME, arg->string->str,
	    -1);
    a = message_arg_none("");
    message_respond(q, id, a, NULL);
    message_arg_free(a);
}

static void
server_remove(MessageQueue q,
	const gchar *name,
	MessageID id,
	const MessageArg *arg,
	gpointer data) {
    Conc *conc = data;
    MessageArg *a;
    GtkTreeIter iter;

    if (arg->type != MESSAGE_ARG_STRING) {
	g_printerr("malformed message in server_remove report!\n");
	return;
    }
    g_assert(gtk_tree_model_get_iter_first(GTK_TREE_MODEL(conc->all_servers),
		&iter));
    do {
	gchar *server = NULL;

	gtk_tree_model_get(GTK_TREE_MODEL(conc->all_servers), &iter,
		SERVERS_COLUMN_NAME, &server, -1);
	if (server == NULL) {
	    continue;
	}
	if (strcmp(arg->string->str, server) == 0) {
	    gtk_list_store_remove(conc->all_servers, &iter);
	    g_free(server);
	    break;
	}
	g_free(server);
    } while (gtk_tree_model_iter_next(GTK_TREE_MODEL(conc->all_servers), &iter));
    a = message_arg_none("");
    message_respond(q, id, a, NULL);
    message_arg_free(a);
}

static void
server_rename(MessageQueue q,
	const gchar *name,
	MessageID id,
	const MessageArg *arg,
	gpointer data) {
    Conc *conc = data;
    MessageArg *a;
    MessageArg *oldname;
    MessageArg *newname;
    GtkTreeIter iter;

    if (arg->type != MESSAGE_ARG_LIST) {
	g_printerr("malformed message in server_rename report!\n");
	return;
    }
    if (message_arg_list_explode(arg,
		"server", MESSAGE_ARG_STRING, &oldname,
		"newname", MESSAGE_ARG_STRING, &newname,
		NULL, 0, NULL) == FALSE) {
	g_printerr("malformed message in server_rename report!\n");
	return;
    }
    g_assert(gtk_tree_model_get_iter_first(GTK_TREE_MODEL(conc->all_servers),
		&iter));
    do {
	gchar *server = NULL;

	gtk_tree_model_get(GTK_TREE_MODEL(conc->all_servers), &iter,
		SERVERS_COLUMN_NAME, &server, -1);
	if (server == NULL) {
	    continue;
	}
	if (strcmp(oldname->string->str, server) == 0) {
	    gtk_list_store_set(conc->all_servers, &iter,
		SERVERS_COLUMN_NAME, newname->string->str,
		-1);
	    g_free(server);
	    break;
	}
	g_free(server);
    } while (gtk_tree_model_iter_next(GTK_TREE_MODEL(conc->all_servers), &iter));
    a = message_arg_none("");
    message_respond(q, id, a, NULL);
    message_arg_free(a);
}

static void
detach(MessageQueue q,
	const gchar *name,
	MessageID id,
	const MessageArg *arg,
	gpointer data) {
    Conc *conc = data;
    MessageArg *a;

    if (arg->type != MESSAGE_ARG_STRING) {
	g_printerr("malformed message in status report!\n");
	return;
    }
    terminal_detach(conc, arg->string->str);
    a = message_arg_none("");
    message_respond(q, id, a, NULL);
    message_arg_free(a);
}

static void
status_report(MessageQueue q,
	const gchar *name,
	MessageID id,
	const MessageArg *arg,
	gpointer data) {
    Conc *conc = data;
    MessageArg *cgroups, *consoles;
    GList *cgl = NULL;
    GList *cl = NULL;
    GList *l;
    MessageArg *a;

    if (arg->type != MESSAGE_ARG_LIST) {
	g_printerr("malformed message in status report!\n");
	return;
    }
    if (message_arg_list_explode(arg,
		"groups", MESSAGE_ARG_LIST, &cgroups,
		"consoles", MESSAGE_ARG_LIST, &consoles,
		NULL, 0, NULL) == FALSE) {
	g_printerr("malformed message in status report!\n");
	return;
    }
    for (l = cgroups->list; l != NULL; l = g_list_next(l)) {
	MessageArg *item = l->data;
	MessageArg *cgroup, *status, *last_active;
	ConcStatus *cs;

	if (item->type != MESSAGE_ARG_LIST) {
	    g_printerr("malformed console group in status report!\n");
	    continue;
	}
	if (message_arg_list_explode(item,
		    "cgroup", MESSAGE_ARG_STRING, &cgroup,
		    "status", MESSAGE_ARG_UINT8, &status,
		    "last_active", MESSAGE_ARG_UINT64, &last_active,
		    NULL, 0, NULL) == FALSE) {
	    g_printerr("malformed console group in status report!\n");
	    continue;
	}
	cs = g_new0(ConcStatus,1);
	cs->name = cgroup->string->str;
	status_from_arg(status, &cs->status);
	cs->last_active = last_active->uint64;
	cgl = g_list_prepend(cgl, cs);
    }
    for (l = consoles->list; l != NULL; l = g_list_next(l)) {
	MessageArg *item = l->data;
	MessageArg *console, *status, *last_active;
	ConcStatus *cs;

	if (item->type != MESSAGE_ARG_LIST) {
	    g_printerr("malformed console in status report!\n");
	    continue;
	}
	if (message_arg_list_explode(item,
		    "console", MESSAGE_ARG_STRING, &console,
		    "status", MESSAGE_ARG_UINT8, &status,
		    "last_active", MESSAGE_ARG_UINT64, &last_active,
		    NULL, 0, NULL) == FALSE) {
	    g_printerr("malformed console in status report!\n");
	    continue;
	}
	cs = g_new0(ConcStatus,1);
	cs->name = console->string->str;
	status_from_arg(status, &cs->status);
	cs->last_active = last_active->uint64;
	cl = g_list_prepend(cl, cs);
    }
    group_status(conc, cgl, cl);
    for (l = cgl; l != NULL; l = g_list_next(l)) {
	g_free(l->data);
    }
    g_list_free(cgl);
    for (l = cl; l != NULL; l = g_list_next(l)) {
	g_free(l->data);
    }
    g_list_free(cl);
    a = message_arg_none("");
    message_respond(q, id, a, NULL);
    message_arg_free(a);
}

static void
subscribed(MessageQueue q,
	const MessageArg *arg,
	gpointer data) {
    struct subscribed_data *sd = data;
    Conc *conc = sd->conc;
    GError *e = message_arg_error_check(arg);
    GList *consoles = NULL;
    GList *l;

    if (e != NULL) {
	g_printerr("could not subscribe to group %s (%s)\n",
		sd->cgroup, e->message);
	g_clear_error(&e);
	g_free(sd->cgroup);
	g_free(sd);
	return;
    }
    for (l = arg->list; l != NULL; l = g_list_next(l)) {
	MessageArg *a = l->data;

	consoles = g_list_prepend(consoles, a->string->str);
    }
    group_add(conc, sd->cgroup, consoles);
    g_list_free(consoles);
    g_free(sd->cgroup);
    g_free(sd);
}

static void
logged_in(MessageQueue q,
	const MessageArg *arg,
	gpointer data) {
    Conc *conc = data;
    GError *e = message_arg_error_check(arg);
    MessageArg *a;
    MessageArg *cgroups;
    MessageArg *consoles;
    MessageArg *logs;
    MessageArg *servers;
    MessageArg *users;
    struct subscribed_data *sd;
    GList *l;
    GtkTreeIter iter;
    GSList *subscribed_list;

    if (e != NULL) {
	error_dialog(conc, _("Login failed"),
		_("The Conc manager has refused your login attempt (%s)."),
		e->message);
	g_clear_error(&e);
	return;
    }
    if (message_arg_list_explode(arg,
		"cgroups", MESSAGE_ARG_LIST, &cgroups,
		"consoles", MESSAGE_ARG_LIST, &consoles,
		"logs", MESSAGE_ARG_LIST, &logs,
		"servers", MESSAGE_ARG_LIST, &servers,
		"users", MESSAGE_ARG_LIST, &users,
		NULL, 0, NULL) == FALSE) {
	error_dialog(conc, _("Internal error"),
		_("Bad response from manager"));
	return;
    }

    for (l = consoles->list; l != NULL; l = g_list_next(l)) {
	MessageArg *a = l->data;

	if (a->type != MESSAGE_ARG_STRING) {
	    continue;
	}
	gtk_list_store_append(conc->all_consoles, &iter);
	gtk_list_store_set(conc->all_consoles, &iter,
		CONSOLES_COLUMN_NAME, a->string->str,
		-1);
    }

    for (l = cgroups->list; l != NULL; l = g_list_next(l)) {
	MessageArg *a = l->data;

	if (a->type != MESSAGE_ARG_STRING) {
	    continue;
	}
	gtk_list_store_append(conc->all_groups, &iter);
	gtk_list_store_set(conc->all_groups, &iter,
		GROUPS_COLUMN_NAME, a->string->str,
		GROUPS_COLUMN_SUBSCRIBED, FALSE,
		-1);
    }

    for (l = logs->list; l != NULL; l = g_list_next(l)) {
	MessageArg *a = l->data;

	if (a->type != MESSAGE_ARG_STRING) {
	    continue;
	}
	gtk_list_store_append(conc->all_logs, &iter);
	gtk_list_store_set(conc->all_logs, &iter,
		LOGS_COLUMN_NAME, a->string->str,
		-1);
    }

    for (l = servers->list; l != NULL; l = g_list_next(l)) {
	MessageArg *a = l->data;

	if (a->type != MESSAGE_ARG_STRING) {
	    continue;
	}
	gtk_list_store_append(conc->all_servers, &iter);
	gtk_list_store_set(conc->all_servers, &iter,
		LOGS_COLUMN_NAME, a->string->str,
		-1);
    }

    for (l = users->list; l != NULL; l = g_list_next(l)) {
	MessageArg *a = l->data;

	if (a->type != MESSAGE_ARG_STRING) {
	    continue;
	}
	gtk_list_store_append(conc->all_users, &iter);
	gtk_list_store_set(conc->all_users, &iter,
		LOGS_COLUMN_NAME, a->string->str,
		-1);
    }

    subscribed_list = gconf_client_get_list(conc->gconf, GCONF_SUBSCRIBED,
	    GCONF_VALUE_STRING, NULL);
    while (subscribed_list != NULL) {
	sd = g_new0(struct subscribed_data, 1);
	sd->cgroup = subscribed_list->data;
	sd->conc = conc;
	a = message_arg_string("cgroup", sd->cgroup);
	if (message_send(conc->msg_q, CLIENT_MESSAGE_SUBSCRIBE, a, subscribed,
		    sd, &e) == FALSE) {
	    error_dialog(conc, _("Internal error"),
		    _("Failed to send message (%s)"), e->message);
	    g_clear_error(&e);
	}
	message_arg_free(a);
	subscribed_list = g_slist_remove(subscribed_list,
		subscribed_list->data);
    }
}

static void
connection_closed(Mplex mplex,
	gpointer data) {
    Conc *conc = data;

    error_dialog(conc, _("Manager closed connection unexpectedly"),
	    _("The manager process may have crashed, or there may be "
	    "a network problem."));

}

static gboolean
register_handlers(Conc *conc, GError **e) {
    return
	message_handler_register(conc->msg_q,
		CLIENT_MESSAGE_STATUS, status_report, conc, e) &&
	message_handler_register(conc->msg_q,
		CLIENT_MESSAGE_DETACH, detach, conc, e) &&
	message_handler_register(conc->msg_q,
		CLIENT_MESSAGE_CGROUP_CHANGE, cgroup_change, conc, e) &&
	message_handler_register(conc->msg_q,
		CLIENT_MESSAGE_CONFIG_CGROUP_ADDED, cgroup_add, conc, e) &&
	message_handler_register(conc->msg_q,
		CLIENT_MESSAGE_CONFIG_CGROUP_REMOVED, cgroup_remove, conc, e) &&
	message_handler_register(conc->msg_q,
		CLIENT_MESSAGE_CONFIG_CGROUP_RENAMED, cgroup_rename, conc, e) &&
	message_handler_register(conc->msg_q,
		CLIENT_MESSAGE_CONFIG_CONSOLE_ADDED, console_add, conc, e) &&
	message_handler_register(conc->msg_q,
		CLIENT_MESSAGE_CONFIG_CONSOLE_REMOVED, console_remove, conc, e) &&
	message_handler_register(conc->msg_q,
		CLIENT_MESSAGE_CONFIG_CONSOLE_RENAMED, console_rename, conc, e) &&
	message_handler_register(conc->msg_q,
		CLIENT_MESSAGE_CONFIG_LOG_ADDED, log_add, conc, e) &&
	message_handler_register(conc->msg_q,
		CLIENT_MESSAGE_CONFIG_LOG_REMOVED, log_remove, conc, e) &&
	message_handler_register(conc->msg_q,
		CLIENT_MESSAGE_CONFIG_LOG_RENAMED, log_rename, conc, e) &&
	message_handler_register(conc->msg_q,
		CLIENT_MESSAGE_CONFIG_USER_ADDED, user_add, conc, e) &&
	message_handler_register(conc->msg_q,
		CLIENT_MESSAGE_CONFIG_USER_REMOVED, user_remove, conc, e) &&
	message_handler_register(conc->msg_q,
		CLIENT_MESSAGE_CONFIG_USER_RENAMED, user_rename, conc, e) &&
	message_handler_register(conc->msg_q,
		CLIENT_MESSAGE_CONFIG_SERVER_ADDED, server_add, conc, e) &&
	message_handler_register(conc->msg_q,
		CLIENT_MESSAGE_CONFIG_SERVER_REMOVED, server_remove, conc, e) &&
	message_handler_register(conc->msg_q,
		CLIENT_MESSAGE_CONFIG_SERVER_RENAMED, server_rename, conc, e) &&
	TRUE;
}

static void
login_changed(GtkEditable *editable,
	gpointer user_data) {
    GladeXML *login = user_data;
    gboolean login_possible = TRUE;
    GtkWidget *w;
    gchar *s;

    w = glade_xml_get_widget(login, LOGIN_DIALOG_HOST);
    s = gtk_editable_get_chars(GTK_EDITABLE(w), 0, -1);
    if (s[0] == '\0') {
	login_possible = FALSE;
    }
    g_free(s);

    w = glade_xml_get_widget(login, LOGIN_DIALOG_USER);
    s = gtk_editable_get_chars(GTK_EDITABLE(w), 0, -1);
    if (s[0] == '\0') {
	login_possible = FALSE;
    }
    g_free(s);

    w = glade_xml_get_widget(login, LOGIN_DIALOG_LOGIN);
    g_object_set(w, "sensitive", login_possible, NULL);
    if (login_possible) {
	gtk_widget_grab_default(w);
    }
}

static void
login_response(GtkDialog *dialog,
	gint arg1,
	gpointer user_data) {
    Conc *conc = user_data;
    GladeXML *login = glade_get_widget_tree(GTK_WIDGET(dialog));
    GtkWidget *w;
    gchar *host, *user, *pass;
    GError *e = NULL;
    GIOChannel *io;
    GList *l = NULL;
    MessageArg *arg;

    if (arg1 != GTK_RESPONSE_ACCEPT) {
	gtk_main_quit();
	return;
    }
    w = glade_xml_get_widget(login, LOGIN_DIALOG_HOST);
    host = gtk_editable_get_chars(GTK_EDITABLE(w), 0, -1);
    w = glade_xml_get_widget(login, LOGIN_DIALOG_USER);
    user = gtk_editable_get_chars(GTK_EDITABLE(w), 0, -1);
    w = glade_xml_get_widget(login, LOGIN_DIALOG_PASSWORD);
    pass = gtk_editable_get_chars(GTK_EDITABLE(w), 0, -1);
    w = glade_xml_get_widget(login, LOGIN_DIALOG);
    gtk_widget_destroy(w);

    conc->sock = gnet_tcp_socket_connect(host, CONC_PORT_MGR);
    if (conc->sock == NULL) {
	error_dialog(conc, _("Failed to connect to host"),
		_("Failed to connect to to %s. This may be because there is "
		"currently no Conc manager running on this host."),
		host);
	g_free(host);
	g_free(user);
	g_free(pass);
	return;
    }

    io = ssl_client(gnet_tcp_socket_get_io_channel(conc->sock));
    conc->mplex = mplex_new(io,
	    connection_closed, conc, &e);
    if (e != NULL) {
	error_dialog(conc, _("Internal error"),
		_("Failed to start mplex (%s)"), e->message);
	g_clear_error(&e);
	g_free(host);
	g_free(user);
	g_free(pass);
	return;
    }

    io = mplex_open(conc->mplex, CONC_MSG_CHANNEL, NULL, NULL, NULL, &e);
    if (e != NULL) {
	error_dialog(conc, _("Internal error"),
		_("Failed to open channel for message queue (%s)"), e->message);
	g_clear_error(&e);
	g_free(host);
	g_free(user);
	g_free(pass);
	return;
    }

    conc->msg_q = message_queue_new(io, &e);
    if (e != NULL) {
	error_dialog(conc, _("Internal error"),
		_("Failed to open message queue (%s)"), e->message);
	g_clear_error(&e);
	g_free(host);
	g_free(user);
	g_free(pass);
	return;
    }

    if (register_handlers(conc, &e) == FALSE) {
	error_dialog(conc, _("Internal error"),
		_("Failed to register message handler (%s)"), e->message);
	g_clear_error(&e);
	g_free(host);
	g_free(user);
	g_free(pass);
	return;
    }

    message_queue_start(conc->msg_q);
    mplex_start(conc->mplex);

    l = g_list_append(l, message_arg_string("user", user));
    l = g_list_append(l, message_arg_string("password", pass));
    arg = message_arg_list("", l);
    if (message_send(conc->msg_q, CLIENT_MESSAGE_LOGIN, arg, logged_in,
		conc, &e) == FALSE) {
	error_dialog(conc, _("Internal error"),
		_("Failed to send message (%s)"), e->message);
	g_clear_error(&e);
	g_free(host);
	g_free(user);
	g_free(pass);
	return;
    }
    message_arg_free(arg);

    gconf_client_set_string(conc->gconf, GCONF_MANAGER, host, NULL);
    gconf_client_set_string(conc->gconf, GCONF_USER, user, NULL);
    g_free(host);
    g_free(user);
    g_free(pass);
}

static void
login(Conc *conc,
	const gchar *arg_host,
	const gchar *arg_user,
	const gchar *arg_pw) {
    GladeXML *login = glade_xml_new(conc->glade_file, LOGIN_DIALOG, NULL);
    char *manager, *user;

    if (arg_host != NULL) {
	manager = g_strdup(arg_host);
    } else {
	manager = gconf_client_get_string(conc->gconf, GCONF_MANAGER, NULL);
    }
    if (manager != NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(login,
			LOGIN_DIALOG_HOST)), manager);
	g_free(manager);
    }
    if (arg_user != NULL) {
	user = g_strdup(arg_user);
    } else {
	user = gconf_client_get_string(conc->gconf, GCONF_USER, NULL);
    }
    if (user != NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(login,
			LOGIN_DIALOG_USER)), user);
	g_free(user);
    }
    if (arg_pw != NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(login,
			LOGIN_DIALOG_PASSWORD)), arg_pw);
    }
    if (manager != NULL && user == NULL) {
	gtk_widget_grab_focus(glade_xml_get_widget(login, LOGIN_DIALOG_USER));
    }
    if (manager != NULL && user != NULL) {
	gtk_widget_grab_focus(glade_xml_get_widget(login, LOGIN_DIALOG_PASSWORD));
    }
    login_changed(NULL, login);
    glade_xml_signal_connect_data(login, LOGIN_DIALOG_REPONSE,
	    (GCallback)login_response, conc);
    glade_xml_signal_connect_data(login, LOGIN_DIALOG_CHANGED,
	    (GCallback)login_changed, login);
    if (arg_user != NULL && arg_pw != NULL) {
	login_response(GTK_DIALOG(glade_xml_get_widget(login, LOGIN_DIALOG)),
		GTK_RESPONSE_ACCEPT, conc);
    }
}

static void
group_selected(GtkNotebook *notebook,
	GtkNotebookPage *page,
	guint page_num,
	gpointer user_data) {
    Conc *conc = user_data;

    group_select(conc, page_num);
}

int
main(int argc, char *argv[]) {
    Conc *conc = g_new0(Conc, 1);
    gint debug = 0;
    gchar *host = NULL;
    gchar *user = NULL;
    gchar *pw_file = NULL;
    gchar *pw = NULL;
    GOptionEntry options[] = {
	{ "debug", 'd', 0, G_OPTION_ARG_NONE, &debug,
	    N_("Enter debug mode"), NULL },
	{ "host", 'h', 0, G_OPTION_ARG_STRING, &host,
	    N_("Host running manager"), N_("HOST") },
	{ "user", 'u', 0, G_OPTION_ARG_STRING, &user,
	    N_("Conc user name"), N_("USER") },
	{ "pwfile", 'p', 0, G_OPTION_ARG_STRING, &pw_file,
	    N_("File containing password"), N_("FILE") },
	{ NULL }
    };
    GOptionContext *option_context;

    option_context = g_option_context_new(NULL);
    g_option_context_add_main_entries(option_context, options,
	GETTEXT_PACKAGE);
    conc->program = gnome_program_init("conc", PACKAGE_VERSION,
	LIBGNOMEUI_MODULE, argc, argv,
	GNOME_PARAM_GOPTION_CONTEXT, option_context,
	GNOME_PARAM_HUMAN_READABLE_NAME,
	_("Serial console concentrator"),
	GNOME_PARAM_APP_DATADIR, DATADIR,
	LIBGNOMEUI_PARAM_DEFAULT_ICON, CONCDATADIR "/conc.png",
	NULL);
    if (debug == 1) {
	g_log_set_always_fatal(G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL);
    }
    conc->gconf = gconf_client_get_default();
    conc->groups = conc->consoles = conc->terminals = NULL;
    conc->all_consoles = gtk_list_store_new(CONSOLES_COLUMN_LAST,
	    G_TYPE_STRING);
    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(conc->all_consoles),
	    CONSOLES_COLUMN_NAME, GTK_SORT_ASCENDING);
    conc->all_groups = gtk_list_store_new(GROUPS_COLUMN_LAST,
	    G_TYPE_STRING,
	    G_TYPE_BOOLEAN);
    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(conc->all_groups),
	    GROUPS_COLUMN_NAME, GTK_SORT_ASCENDING);
    conc->all_logs = gtk_list_store_new(LOGS_COLUMN_LAST, G_TYPE_STRING);
    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(conc->all_logs),
	    LOGS_COLUMN_NAME, GTK_SORT_ASCENDING);
    conc->all_servers = gtk_list_store_new(SERVERS_COLUMN_LAST, G_TYPE_STRING);
    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(conc->all_servers),
	    SERVERS_COLUMN_NAME, GTK_SORT_ASCENDING);
    conc->all_users = gtk_list_store_new(USERS_COLUMN_LAST, G_TYPE_STRING);
    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(conc->all_users),
	    USERS_COLUMN_NAME, GTK_SORT_ASCENDING);
    conc->matrix = matrix_new();
    conc->ok_img = gdk_pixbuf_new_from_file(CONCDATADIR "/ok.png", NULL);
    conc->failed_img = gdk_pixbuf_new_from_file(CONCDATADIR "/failed.png",
	    NULL);
    conc->active_img = gdk_pixbuf_new_from_file(CONCDATADIR "/active.png", NULL);
    conc->glade_file = CONCDATADIR "/conc.glade";
    conc->main_window = glade_xml_new(conc->glade_file, MAIN_WINDOW, NULL);
    conc->notebook = GTK_NOTEBOOK(glade_xml_get_widget(conc->main_window,
	    MAIN_WINDOW_NOTEBOOK));
    gtk_notebook_remove_page(conc->notebook, 0);
    conc->menu_view_console =
	GTK_MENU_ITEM(glade_xml_get_widget(conc->main_window,
		    MENU_VIEW_CONSOLE));
    g_object_set(conc->menu_view_console, "sensitive", FALSE, NULL);
    glade_xml_signal_connect_data(conc->main_window, MAIN_PAGE_SWITCH,
	    (GCallback)group_selected, conc);
    glade_xml_signal_connect(conc->main_window, "gtk_main_quit",
	    (GCallback)gtk_main_quit);
    glade_xml_signal_connect_data(conc->main_window, MENU_CONC_CHANGE_PASSWORD,
	    (GCallback)menu_conc_change_password, conc);
    glade_xml_signal_connect_data(conc->main_window, MENU_EDIT_SUBSCRIBED,
	    (GCallback)menu_edit_subscribed, conc);
    glade_xml_signal_connect_data(conc->main_window, MENU_EDIT_CONFIGURATION,
	    (GCallback)menu_edit_configuration, conc);
    glade_xml_signal_connect_data(conc->main_window, MENU_VIEW_CONSOLE,
	    (GCallback)menu_view_console, conc);
    glade_xml_signal_connect_data(conc->main_window, MENU_VIEW_LOG,
	    (GCallback)menu_view_log, conc);
    glade_xml_signal_connect_data(conc->main_window, MENU_HELP_CONTENTS,
	    (GCallback)menu_help_contents, conc);
    glade_xml_signal_connect_data(conc->main_window, MENU_HELP_ABOUT,
	    (GCallback)menu_help_about, conc);
    gtk_about_dialog_set_email_hook(about_email, NULL, NULL);
    gtk_about_dialog_set_url_hook(about_url, NULL, NULL);
    if (pw_file != NULL) {
	GError *e = NULL;
	gint i;

	if (g_file_get_contents(pw_file, &pw, NULL, &e) == FALSE) {
	    error_dialog(conc, _("Failed to open password file"),
		    _("Failed to open %s (%s)"), pw_file, e->message);
	}
	for (i = 0; pw[i] != '\0'; i++) {
	    if (pw[i] == '\n') {
		pw[i] = '\0';
		break;
	    }
	}
    }
    login(conc, host, user, pw);
    if (pw) {
	/*FIXME: should blank pw*/
	g_free(pw);
    }
    gtk_main();
    exit(EXIT_SUCCESS);
}
