#include "conc.h"
#include "error.h"
#include "configuration.h"
#include "config-cgroups.h"
#include "config-consoles.h"
#include "config-users.h"
#include "config-servers.h"
#include "config-logs.h"

typedef struct _ConfigurationDialog {
    Conc *conc;
    GladeXML *xml;
} ConfigurationDialog;

/* 
 * Glade widget names:
 */
#define CONFIG_DIALOG			"configuration dialog"
#define CONFIG_DIALOG_CONSOLES_TV	"consoles treeview"
#define CONFIG_DIALOG_CONSOLES_ADD	"consoles add"
#define CONFIG_DIALOG_CONSOLES_EDIT	"consoles edit"
#define CONFIG_DIALOG_CONSOLES_REMOVE	"consoles remove"
#define CONFIG_DIALOG_CGROUPS_TV	"cgroups treeview"
#define CONFIG_DIALOG_CGROUPS_ADD	"cgroups add"
#define CONFIG_DIALOG_CGROUPS_EDIT	"cgroups edit"
#define CONFIG_DIALOG_CGROUPS_REMOVE	"cgroups remove"
#define CONFIG_DIALOG_LOGS_TV		"logs treeview"
#define CONFIG_DIALOG_LOGS_ADD		"logs add"
#define CONFIG_DIALOG_LOGS_EDIT		"logs edit"
#define CONFIG_DIALOG_LOGS_REMOVE	"logs remove"
#define CONFIG_DIALOG_SERVERS_TV	"servers treeview"
#define CONFIG_DIALOG_SERVERS_ADD	"servers add"
#define CONFIG_DIALOG_SERVERS_EDIT	"servers edit"
#define CONFIG_DIALOG_SERVERS_REMOVE	"servers remove"
#define CONFIG_DIALOG_USERS_TV		"users treeview"
#define CONFIG_DIALOG_USERS_ADD		"users add"
#define CONFIG_DIALOG_USERS_EDIT	"users edit"
#define CONFIG_DIALOG_USERS_REMOVE	"users remove"

/*
 * Glade handler names:
 */
#define CONFIG_DIALOG_RESPONSE		"response"
#define CONFIG_DIALOG_CONSOLES_CHANGED	"consoles changed"
#define CONFIG_DIALOG_ADD_CONSOLE	"add console"
#define CONFIG_DIALOG_EDIT_CONSOLE	"edit console"
#define CONFIG_DIALOG_REMOVE_CONSOLE	"remove console"
#define CONFIG_DIALOG_CGROUPS_CHANGED	"cgroups changed"
#define CONFIG_DIALOG_ADD_CGROUP	"add cgroup"
#define CONFIG_DIALOG_EDIT_CGROUP	"edit cgroup"
#define CONFIG_DIALOG_REMOVE_CGROUP	"remove cgroup"
#define CONFIG_DIALOG_LOGS_CHANGED	"logs changed"
#define CONFIG_DIALOG_ADD_LOG		"add log"
#define CONFIG_DIALOG_EDIT_LOG		"edit log"
#define CONFIG_DIALOG_REMOVE_LOG	"remove log"
#define CONFIG_DIALOG_SERVERS_CHANGED	"servers changed"
#define CONFIG_DIALOG_ADD_SERVER	"add server"
#define CONFIG_DIALOG_EDIT_SERVER	"edit server"
#define CONFIG_DIALOG_REMOVE_SERVER	"remove server"
#define CONFIG_DIALOG_USERS_CHANGED	"users changed"
#define CONFIG_DIALOG_ADD_USER		"add user"
#define CONFIG_DIALOG_EDIT_USER		"edit user"
#define CONFIG_DIALOG_REMOVE_USER	"remove user"


static gchar * /* caller owned */
configuration_selected_treeview(ConfigurationDialog *cd,
	GtkListStore *list,
	gint column,
	GtkTreeView *tv) {
    GtkTreeSelection *selection = gtk_tree_view_get_selection(tv);
    GtkTreeIter iter;
    gchar *name = NULL;

    if (gtk_tree_selection_get_selected(selection, NULL, &iter) == FALSE) {
	return NULL;
    }
    gtk_tree_model_get(GTK_TREE_MODEL(list), &iter, column, &name, -1);
    return name;
}



/**** Consoles */

static gchar * /* caller owned */
configuration_selected_console(ConfigurationDialog *cd) {
    return configuration_selected_treeview(cd, cd->conc->all_consoles,
	    CONSOLES_COLUMN_NAME,
	    GTK_TREE_VIEW(glade_xml_get_widget(cd->xml,
		    CONFIG_DIALOG_CONSOLES_TV)));
}

static void
configuration_console_add(GtkButton *button,
	gpointer user_data) {
    ConfigurationDialog *cd = user_data;

    config_consoles_add(cd->conc);
}

static void
configuration_console_edit(GtkButton *button,
	gpointer user_data) {
    ConfigurationDialog *cd = user_data;
    gchar *console = configuration_selected_console(cd);

    config_consoles_edit(cd->conc, console);
    g_free(console);
}

static void
configuration_console_remove(GtkButton *button,
	gpointer user_data) {
    ConfigurationDialog *cd = user_data;
    gchar *console = configuration_selected_console(cd);

    config_consoles_remove(cd->conc, console);
    g_free(console);
}




/**** Console Groups */

static gchar * /* caller owned */
configuration_selected_cgroup(ConfigurationDialog *cd) {
    return configuration_selected_treeview(cd, cd->conc->all_groups,
	    GROUPS_COLUMN_NAME,
	    GTK_TREE_VIEW(glade_xml_get_widget(cd->xml,
		    CONFIG_DIALOG_CGROUPS_TV)));
}

static void
configuration_cgroup_add(GtkButton *button,
	gpointer user_data) {
    ConfigurationDialog *cd = user_data;

    config_cgroups_add(cd->conc);
}

static void
configuration_cgroup_edit(GtkButton *button,
	gpointer user_data) {
    ConfigurationDialog *cd = user_data;
    gchar *cgroup = configuration_selected_cgroup(cd);

    config_cgroups_edit(cd->conc, cgroup);
    g_free(cgroup);
}

static void
configuration_cgroup_remove(GtkButton *button,
	gpointer user_data) {
    ConfigurationDialog *cd = user_data;
    gchar *cgroup = configuration_selected_cgroup(cd);

    config_cgroups_remove(cd->conc, cgroup);
    g_free(cgroup);
}



/**** Logs */

static gchar * /* caller owned */
configuration_selected_log(ConfigurationDialog *cd) {
    return configuration_selected_treeview(cd, cd->conc->all_logs,
	    LOGS_COLUMN_NAME,
	    GTK_TREE_VIEW(glade_xml_get_widget(cd->xml,
		    CONFIG_DIALOG_LOGS_TV)));
}

static void
configuration_log_add(GtkButton *button,
	gpointer user_data) {
    ConfigurationDialog *cd = user_data;

    config_logs_add(cd->conc);
}

static void
configuration_log_edit(GtkButton *button,
	gpointer user_data) {
    ConfigurationDialog *cd = user_data;
    gchar *log = configuration_selected_log(cd);

    config_logs_edit(cd->conc, log);
    g_free(log);
}

static void
configuration_log_remove(GtkButton *button,
	gpointer user_data) {
    ConfigurationDialog *cd = user_data;
    gchar *log = configuration_selected_log(cd);

    config_logs_remove(cd->conc, log);
    g_free(log);
}




/**** Servers */

static gchar * /* caller owned */
configuration_selected_server(ConfigurationDialog *cd) {
    return configuration_selected_treeview(cd, cd->conc->all_servers,
	    SERVERS_COLUMN_NAME,
	    GTK_TREE_VIEW(glade_xml_get_widget(cd->xml,
		    CONFIG_DIALOG_SERVERS_TV)));
}

static void
configuration_server_add(GtkButton *button,
	gpointer user_data) {
    ConfigurationDialog *cd = user_data;

    config_servers_add(cd->conc);
}

static void
configuration_server_edit(GtkButton *button,
	gpointer user_data) {
    ConfigurationDialog *cd = user_data;
    gchar *server = configuration_selected_server(cd);

    config_servers_edit(cd->conc, server);
    g_free(server);
}

static void
configuration_server_remove(GtkButton *button,
	gpointer user_data) {
    ConfigurationDialog *cd = user_data;
    gchar *server = configuration_selected_server(cd);

    config_servers_remove(cd->conc, server);
    g_free(server);
}



/**** Users */

static gchar * /* caller owned */
configuration_selected_user(ConfigurationDialog *cd) {
    return configuration_selected_treeview(cd, cd->conc->all_users,
	    USERS_COLUMN_NAME,
	    GTK_TREE_VIEW(glade_xml_get_widget(cd->xml,
		    CONFIG_DIALOG_USERS_TV)));
}

static void
configuration_user_add(GtkButton *button,
	gpointer user_data) {
    ConfigurationDialog *cd = user_data;

    config_users_add(cd->conc);
}

static void
configuration_user_edit(GtkButton *button,
	gpointer user_data) {
    ConfigurationDialog *cd = user_data;
    gchar *user = configuration_selected_user(cd);

    config_users_edit(cd->conc, user);
    g_free(user);
}

static void
configuration_user_remove(GtkButton *button,
	gpointer user_data) {
    ConfigurationDialog *cd = user_data;
    gchar *user = configuration_selected_user(cd);

    config_users_remove(cd->conc, user);
    g_free(user);
}




static void
configuration_response(GtkDialog *dialog,
	gint arg1,
	gpointer user_data) {
    ConfigurationDialog *cd = user_data;

    gtk_widget_destroy(glade_xml_get_widget(cd->xml, CONFIG_DIALOG));
    g_free(cd);
}

static void
configuration_treeview_changed(ConfigurationDialog *cd,
	GtkTreeView *tv,
	const gchar *edit_but,
	const gchar *remove_but) {
    GtkWidget *edit = glade_xml_get_widget(cd->xml, edit_but);
    GtkWidget *remove = glade_xml_get_widget(cd->xml, remove_but);
    
    if (gtk_tree_selection_count_selected_rows(gtk_tree_view_get_selection(tv))
	    == 0) {
	g_object_set(edit, "sensitive", FALSE, NULL);
	g_object_set(remove, "sensitive", FALSE, NULL);
    } else {
	g_object_set(edit, "sensitive", TRUE, NULL);
	g_object_set(remove, "sensitive", TRUE, NULL);
    }
}

static void
configuration_consoles_changed(GtkTreeView *tv,
	gpointer user_data) {
    ConfigurationDialog *cd = user_data;

    configuration_treeview_changed(cd, tv,
	    CONFIG_DIALOG_CONSOLES_EDIT,
	    CONFIG_DIALOG_CONSOLES_REMOVE);
}

static void
configuration_cgroups_changed(GtkTreeView *tv,
	gpointer user_data) {
    ConfigurationDialog *cd = user_data;

    configuration_treeview_changed(cd, tv,
	    CONFIG_DIALOG_CGROUPS_EDIT,
	    CONFIG_DIALOG_CGROUPS_REMOVE);
}

static void
configuration_logs_changed(GtkTreeView *tv,
	gpointer user_data) {
    ConfigurationDialog *cd = user_data;

    configuration_treeview_changed(cd, tv,
	    CONFIG_DIALOG_LOGS_EDIT,
	    CONFIG_DIALOG_LOGS_REMOVE);
}

static void
configuration_servers_changed(GtkTreeView *tv,
	gpointer user_data) {
    ConfigurationDialog *cd = user_data;

    configuration_treeview_changed(cd, tv,
	    CONFIG_DIALOG_SERVERS_EDIT,
	    CONFIG_DIALOG_SERVERS_REMOVE);
}

static void
configuration_users_changed(GtkTreeView *tv,
	gpointer user_data) {
    ConfigurationDialog *cd = user_data;

    configuration_treeview_changed(cd, tv,
	    CONFIG_DIALOG_USERS_EDIT,
	    CONFIG_DIALOG_USERS_REMOVE);
}

extern void
configuration_show(Conc *conc) {
    ConfigurationDialog *cd = g_new0(struct _ConfigurationDialog, 1);
    GtkCellRenderer *text_rend;
    GtkTreeViewColumn *col;
    GtkTreeView *tv;

    cd->conc = conc;
    cd->xml = glade_xml_new(conc->glade_file, CONFIG_DIALOG, NULL);
    glade_xml_signal_connect_data(cd->xml, CONFIG_DIALOG_RESPONSE,
	    (GCallback)configuration_response, cd);
    text_rend = gtk_cell_renderer_text_new();

    /* consoles */
    tv = GTK_TREE_VIEW(glade_xml_get_widget(cd->xml,
		CONFIG_DIALOG_CONSOLES_TV));
    g_object_set(tv,
	    "model", conc->all_consoles,
	    "rules-hint", TRUE,
	    "headers-clickable", TRUE,
	    "reorderable", TRUE,
	    "enable-search", TRUE,
	    "search-column", CONSOLES_COLUMN_NAME,
	    NULL);
    col = gtk_tree_view_column_new_with_attributes(_("Console"), text_rend,
	    "text", CONSOLES_COLUMN_NAME, NULL);
    gtk_tree_view_append_column(tv, col);
    glade_xml_signal_connect_data(cd->xml, CONFIG_DIALOG_CONSOLES_CHANGED,
	    (GCallback)configuration_consoles_changed, cd);
    configuration_consoles_changed(tv, cd);
    glade_xml_signal_connect_data(cd->xml, CONFIG_DIALOG_ADD_CONSOLE,
	    (GCallback)configuration_console_add, cd);
    glade_xml_signal_connect_data(cd->xml, CONFIG_DIALOG_EDIT_CONSOLE,
	    (GCallback)configuration_console_edit, cd);
    glade_xml_signal_connect_data(cd->xml, CONFIG_DIALOG_REMOVE_CONSOLE,
	    (GCallback)configuration_console_remove, cd);

    /* console groups */
    tv = GTK_TREE_VIEW(glade_xml_get_widget(cd->xml,
		CONFIG_DIALOG_CGROUPS_TV));
    g_object_set(tv,
	    "model", conc->all_groups,
	    "rules-hint", TRUE,
	    "headers-clickable", TRUE,
	    "reorderable", TRUE,
	    "enable-search", TRUE,
	    "search-column", GROUPS_COLUMN_NAME,
	    NULL);
    col = gtk_tree_view_column_new_with_attributes(_("Console Group"),
	    text_rend, "text", GROUPS_COLUMN_NAME, NULL);
    g_object_set(col,
	    "reorderable", TRUE,
	    "sort-indicator", TRUE,
	    NULL);
    gtk_tree_view_append_column(tv, col);
    glade_xml_signal_connect_data(cd->xml, CONFIG_DIALOG_CGROUPS_CHANGED,
	    (GCallback)configuration_cgroups_changed, cd);
    configuration_cgroups_changed(tv, cd);
    glade_xml_signal_connect_data(cd->xml, CONFIG_DIALOG_ADD_CGROUP,
	    (GCallback)configuration_cgroup_add, cd);
    glade_xml_signal_connect_data(cd->xml, CONFIG_DIALOG_EDIT_CGROUP,
	    (GCallback)configuration_cgroup_edit, cd);
    glade_xml_signal_connect_data(cd->xml, CONFIG_DIALOG_REMOVE_CGROUP,
	    (GCallback)configuration_cgroup_remove, cd);

    /* logs */
    tv = GTK_TREE_VIEW(glade_xml_get_widget(cd->xml,
		CONFIG_DIALOG_LOGS_TV));
    g_object_set(tv,
	    "model", conc->all_logs,
	    "rules-hint", TRUE,
	    "headers-clickable", TRUE,
	    "reorderable", TRUE,
	    "enable-search", TRUE,
	    "search-column", LOGS_COLUMN_NAME,
	    NULL);
    col = gtk_tree_view_column_new_with_attributes(_("Log"), text_rend,
	    "text", LOGS_COLUMN_NAME, NULL);
    g_object_set(col,
	    "reorderable", TRUE,
	    "sort-indicator", TRUE,
	    NULL);
    gtk_tree_view_append_column(tv, col);
    glade_xml_signal_connect_data(cd->xml, CONFIG_DIALOG_LOGS_CHANGED,
	    (GCallback)configuration_logs_changed, cd);
    configuration_logs_changed(tv, cd);
    glade_xml_signal_connect_data(cd->xml, CONFIG_DIALOG_ADD_LOG,
	    (GCallback)configuration_log_add, cd);
    glade_xml_signal_connect_data(cd->xml, CONFIG_DIALOG_EDIT_LOG,
	    (GCallback)configuration_log_edit, cd);
    glade_xml_signal_connect_data(cd->xml, CONFIG_DIALOG_REMOVE_LOG,
	    (GCallback)configuration_log_remove, cd);

    /* servers */
    tv = GTK_TREE_VIEW(glade_xml_get_widget(cd->xml,
		CONFIG_DIALOG_SERVERS_TV));
    g_object_set(tv,
	    "model", conc->all_servers,
	    "rules-hint", TRUE,
	    "headers-clickable", TRUE,
	    "reorderable", TRUE,
	    "enable-search", TRUE,
	    "search-column", SERVERS_COLUMN_NAME,
	    NULL);
    col = gtk_tree_view_column_new_with_attributes(_("Server"), text_rend,
	    "text", SERVERS_COLUMN_NAME, NULL);
    g_object_set(col,
	    "reorderable", TRUE,
	    "sort-indicator", TRUE,
	    NULL);
    gtk_tree_view_append_column(tv, col);
    glade_xml_signal_connect_data(cd->xml, CONFIG_DIALOG_SERVERS_CHANGED,
	    (GCallback)configuration_servers_changed, cd);
    configuration_servers_changed(tv, cd);
    glade_xml_signal_connect_data(cd->xml, CONFIG_DIALOG_ADD_SERVER,
	    (GCallback)configuration_server_add, cd);
    glade_xml_signal_connect_data(cd->xml, CONFIG_DIALOG_EDIT_SERVER,
	    (GCallback)configuration_server_edit, cd);
    glade_xml_signal_connect_data(cd->xml, CONFIG_DIALOG_REMOVE_SERVER,
	    (GCallback)configuration_server_remove, cd);

    /* users */
    tv = GTK_TREE_VIEW(glade_xml_get_widget(cd->xml,
		CONFIG_DIALOG_USERS_TV));
    g_object_set(tv,
	    "model", conc->all_users,
	    "rules-hint", TRUE,
	    "headers-clickable", TRUE,
	    "reorderable", TRUE,
	    "enable-search", TRUE,
	    "search-column", USERS_COLUMN_NAME,
	    NULL);
    col = gtk_tree_view_column_new_with_attributes(_("User"), text_rend,
	    "text", USERS_COLUMN_NAME, NULL);
    g_object_set(col,
	    "reorderable", TRUE,
	    "sort-indicator", TRUE,
	    NULL);
    gtk_tree_view_append_column(tv, col);
    glade_xml_signal_connect_data(cd->xml, CONFIG_DIALOG_USERS_CHANGED,
	    (GCallback)configuration_users_changed, cd);
    configuration_users_changed(tv, cd);
    glade_xml_signal_connect_data(cd->xml, CONFIG_DIALOG_ADD_USER,
	    (GCallback)configuration_user_add, cd);
    glade_xml_signal_connect_data(cd->xml, CONFIG_DIALOG_EDIT_USER,
	    (GCallback)configuration_user_edit, cd);
    glade_xml_signal_connect_data(cd->xml, CONFIG_DIALOG_REMOVE_USER,
	    (GCallback)configuration_user_remove, cd);

}

