#ifndef __CLIENT_MESSAGE_H
#define __CLIENT_MESSAGE_H

#define CLIENT_MESSAGE_LOGIN "c_LOGIN"
/*
 * DIRECTION:
 *	client -> manager
 *
 * MESSAGE:
 *	(list) ->
 *	    user(string)
 *	    password(string)
 *
 * RESPONSE:
 *	(list) ->
 *	    cgroups(list)
 *		(string)
 *		...
*	    consoles(list)
*		(string)
*		...
 *	    logs(list)
 *		(string)
 *		...
 *	    servers(list)
 *		(string)
 *		...
 *	    users(list)
 *		(string)
 *		...
 *
 * ERROR:
 *	0 - bad user/password.
 */

#define CLIENT_MESSAGE_ATTACH "c_ATTACH"
/*
 * DIRECTION:
 *	client -> manager
 *
 * MESSAGE:
 *	console(string)
 *
 * RESPONSE:
 *	channel(uint16)
 *
 * ERROR:
 *	0 - no such console
 *	1 - permission denied
 *	2 - console unavailable
 *
 */

#define CLIENT_MESSAGE_DETACH "c_DETACH"
/*
 * DIRECTION:
 *	client <-> manager
 *
 * MESSAGE:
 *	console(string)
 *
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *	0 - no such console
 *	1 - not attached
 *
 */

#define CLIENT_MESSAGE_CONSOLE_LOGS "c_CONSOLE_LOGS"
/*
 * DIRECTION:
 *	client <-> manager
 *
 * MESSAGE:
 *	console(string)
 *
 * RESPONSE:
 *	consoles(list)
 *	    (string)
 *	    ...
 *
 * ERROR:
 *	0 - no such console
 *
 */

#define CLIENT_MESSAGE_LOG "c_LOG"
/*
 * DIRECTION:
 *	client -> manager
 *
 * MESSAGE:
 *	query(list)
 *	    log(string)
 *	    start_time(uint64)
 *	    end_time(uint64)
 *	    max_lines(uint32)
 *	    console(string) - optional
 *
 * RESPONSE:
 *	channel(uint16)
 *
 * ERROR:
 *	0 - no such log
 *	1 - permission denied
 *	2 - log unavailable
 *
 */

#define CLIENT_MESSAGE_SUBSCRIBE "c_SUB"
/*
 * DIRECTION:
 *	client -> manager
 *
 * MESSAGE:
 *	cgroup(string)
 *
 * REPONSE:
 *	consoles(list)
 *	    (string)
 *	    ...
 *
 * ERROR:
 *	0 - no such cgroup
 *	1 - permission denied
 *
 */

#define CLIENT_MESSAGE_UNSUBSCRIBE "c_UNSUB"
/*
 * DIRECTION:
 *	client -> manager
 *
 * MESSAGE:
 *	cgroup(string)
 *
 * REPONSE:
 *	(none)
 *
 * ERROR:
 *	0 - no such cgroup
 *	1 - not subscribed
 *
 */

#define CLIENT_MESSAGE_CGROUP_CHANGE "c_CGROUP_CHANGE"
/*
 * DIRECTION:
 *	manager -> client
 *
 * MESSAGE:
 *	(list)
 *	    cgroup(string)
 *	    console(string)
 *	    status(uint8) - 0 = removed, 1 = added
 *
 * RESPONSE:
 *	(none)
 *
 */

#define CLIENT_MESSAGE_CURRSUB "c_CURRENT"
/*
 * DIRECTION:
 *	client -> manager
 * 
 * MESSAGE:
 *	cgroup(string)
 *
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *	0 - no such cgroup
 *	1 - not subscribed
 *
 */

#define CLIENT_MESSAGE_STATUS "c_STATUS"
/*
 * DIRECTION:
 *	manager -> client
 *
 * MESSAGE:
 *	groups(list)
 *	    (list)
 *		cgroup(string)
 *		status(uint8)
 *		last_active(uint64)
 *	    ..
 *	consoles(list)
 *	    (list)
 *		console(string)
 *		status(uint8)
 *		last_active(uint64)
 *	    ..
 * 
 * RESPONSE:
 *	(none)
 * 
 * ERROR:
 *
 */

#define CLIENT_MESSAGE_CHANGE_PASSWORD "c_CHANGEPW"
/*
 * DIRECTION:
 *	client -> manager
 *
 * MESSAGE:
 *	(list)
 *	    old(string)
 *	    new(string)
 *
 * RESPONSE:
 *	(none)
 * 
 * ERROR:
 *	0 - incorrect old password
 *	1 - invalid new password
 *
 */

#define CLIENT_MESSAGE_CONFIG_CGROUP_SPEC "c_CONFIGCGROUPSPEC"
/*
 * DIRECTION:
 *	client->manager
 *
 * MESSAGE:
 *	name(string)
 * 
 * RESPONSE:
 *	(list)
 *	    [SpecCgroup]
 *
 * ERROR:
 *	0 - invalid cgroup
 *
 */

#define CLIENT_MESSAGE_CONFIG_CGROUP_ADD "c_CONFIGCGROUPADD"
/*
 * DIRECTION:
 *	client->manager
 *
 * MESSAGE:
 *	(list)
 *	    [SpecCgroup]
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *	0 - invalid console group specification
 *
 */

#define CLIENT_MESSAGE_CONFIG_CGROUP_EDIT "c_CONFIGCGROUPEDIT"
/*
 * DIRECTION:
 *	client->manager
 *
 * MESSAGE:
 *	(list)
 *	    name(string)
 *	    spec(list)
 *		[SpecCgroup]
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *	0 - invalid cgroup specification
 *
 */

#define CLIENT_MESSAGE_CONFIG_CGROUP_REMOVE "c_CONFIGCGROUPREMOVE"
/*
 * DIRECTION:
 *	client->manager
 *
 * MESSAGE:
 *	name(string)
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *	0 - invalid console group name
 *
 */

#define CLIENT_MESSAGE_CONFIG_CGROUP_ADDED "c_CONFIGCGROUPADDED"
/*
 * DIRECTION:
 *	manager->client
 *
 * MESSAGE:
 *	cgroup(string)
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *
 */

#define CLIENT_MESSAGE_CONFIG_CGROUP_REMOVED "c_CONFIGCGROUPREMOVED"
/*
 * DIRECTION:
 *	manager->client
 *
 * MESSAGE:
 *	cgroup(string)
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *
 */

#define CLIENT_MESSAGE_CONFIG_CGROUP_RENAMED "c_CONFIGCGROUPRENAMED"
/*
 * DIRECTION:
 *	manager->client
 *
 * MESSAGE:
 *	(list)
 *	    cgroup(string)
 *	    newname(string)
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *
 */

#define CLIENT_MESSAGE_CONFIG_CONSOLE_SPEC "c_CONFIGCONSOLESPEC"
/*
 * DIRECTION:
 *	client->manager
 *
 * MESSAGE:
 *	name(string)
 * 
 * RESPONSE:
 *	(list)
 *	    [SpecConsole]
 *
 * ERROR:
 *	0 - invalid console
 *
 */

#define CLIENT_MESSAGE_CONFIG_CONSOLE_ADD "c_CONFIGCONSOLEADD"
/*
 * DIRECTION:
 *	client->manager
 *
 * MESSAGE:
 *	(list)
 *	    [SpecConsole]
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *	0 - invalid console specification
 *
 */

#define CLIENT_MESSAGE_CONFIG_CONSOLE_EDIT "c_CONFIGCONSOLEEDIT"
/*
 * DIRECTION:
 *	client->manager
 *
 * MESSAGE:
 *	(list)
 *	    name(string)
 *	    spec(list)
 *		[SpecConsole]
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *	0 - invalid console specification
 *
 */

#define CLIENT_MESSAGE_CONFIG_CONSOLE_REMOVE "c_CONFIGCONSOLEREMOVE"
/*
 * DIRECTION:
 *	client->manager
 *
 * MESSAGE:
 *	name(string)
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *	0 - invalid console name
 *
 */

#define CLIENT_MESSAGE_CONFIG_CONSOLE_ADDED "c_CONFIGCONSOLEADDED"
/*
 * DIRECTION:
 *	manager->client
 *
 * MESSAGE:
 *	console(string)
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *
 */

#define CLIENT_MESSAGE_CONFIG_CONSOLE_REMOVED "c_CONFIGCONSOLEREMOVED"
/*
 * DIRECTION:
 *	manager->client
 *
 * MESSAGE:
 *	console(string)
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *
 */

#define CLIENT_MESSAGE_CONFIG_CONSOLE_RENAMED "c_CONFIGCONSOLERENAMED"
/*
 * DIRECTION:
 *	manager->client
 *
 * MESSAGE:
 *	(list)
 *	    console(string)
 *	    newname(string)
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *
 */

#define CLIENT_MESSAGE_CONFIG_LOG_SPEC "c_CONFIGLOGSPEC"
/*
 * DIRECTION:
 *	client->manager
 *
 * MESSAGE:
 *	name(string)
 * 
 * RESPONSE:
 *	(list)
 *	    [SpecLog]
 *
 * ERROR:
 *	0 - invalid log
 *
 */

#define CLIENT_MESSAGE_CONFIG_LOG_ADD "c_CONFIGLOGADD"
/*
 * DIRECTION:
 *	client->manager
 *
 * MESSAGE:
 *	(list)
 *	    [SpecLog]
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *	0 - invalid log specification
 *
 */

#define CLIENT_MESSAGE_CONFIG_LOG_EDIT "c_CONFIGLOGEDIT"
/*
 * DIRECTION:
 *	client->manager
 *
 * MESSAGE:
 *	(list)
 *	    name(string)
 *	    spec(list)
 *		[SpecLog]
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *	0 - invalid log specification
 *
 */

#define CLIENT_MESSAGE_CONFIG_LOG_REMOVE "c_CONFIGLOGREMOVE"
/*
 * DIRECTION:
 *	client->manager
 *
 * MESSAGE:
 *	name(string)
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *	0 - invalid log name
 *
 */

#define CLIENT_MESSAGE_CONFIG_LOG_ADDED "c_CONFIGLOGADDED"
/*
 * DIRECTION:
 *	manager->client
 *
 * MESSAGE:
 *	log(string)
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *
 */

#define CLIENT_MESSAGE_CONFIG_LOG_REMOVED "c_CONFIGLOGREMOVED"
/*
 * DIRECTION:
 *	manager->client
 *
 * MESSAGE:
 *	log(string)
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *
 */

#define CLIENT_MESSAGE_CONFIG_LOG_RENAMED "c_CONFIGLOGRENAMED"
/*
 * DIRECTION:
 *	manager->client
 *
 * MESSAGE:
 *	(list)
 *	    log(string)
 *	    newname(string)
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *
 */




#define CLIENT_MESSAGE_CONFIG_USER_SPEC "c_CONFIGUSERSPEC"
/*
 * DIRECTION:
 *	client->manager
 *
 * MESSAGE:
 *	name(string)
 * 
 * RESPONSE:
 *	(list)
 *	    [SpecUser]
 *
 * ERROR:
 *	0 - invalid user
 *
 */

#define CLIENT_MESSAGE_CONFIG_USER_ADD "c_CONFIGUSERADD"
/*
 * DIRECTION:
 *	client->manager
 *
 * MESSAGE:
 *	(list)
 *	    [SpecUser]
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *	0 - invalid user specification
 *
 */

#define CLIENT_MESSAGE_CONFIG_USER_EDIT "c_CONFIGUSEREDIT"
/*
 * DIRECTION:
 *	client->manager
 *
 * MESSAGE:
 *	(list)
 *	    name(string)
 *	    spec(list)
 *		[SpecUser]
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *	0 - invalid user specification
 *
 */

#define CLIENT_MESSAGE_CONFIG_USER_REMOVE "c_CONFIGUSERREMOVE"
/*
 * DIRECTION:
 *	client->manager
 *
 * MESSAGE:
 *	name(string)
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *	0 - invalid user name
 *
 */

#define CLIENT_MESSAGE_CONFIG_USER_ADDED "c_CONFIGUSERADDED"
/*
 * DIRECTION:
 *	manager->client
 *
 * MESSAGE:
 *	user(string)
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *
 */

#define CLIENT_MESSAGE_CONFIG_USER_REMOVED "c_CONFIGUSERREMOVED"
/*
 * DIRECTION:
 *	manager->client
 *
 * MESSAGE:
 *	user(string)
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *
 */

#define CLIENT_MESSAGE_CONFIG_USER_RENAMED "c_CONFIGUSERRENAMED"
/*
 * DIRECTION:
 *	manager->client
 *
 * MESSAGE:
 *	(list)
 *	    user(string)
 *	    newname(string)
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *
 */


#define CLIENT_MESSAGE_CONFIG_SERVER_SPEC "c_CONFIGSERVERSPEC"
/*
 * DIRECTION:
 *	client->manager
 *
 * MESSAGE:
 *	name(string)
 * 
 * RESPONSE:
 *	(list)
 *	    [SpecServer]
 *
 * ERROR:
 *	0 - invalid server
 *
 */

#define CLIENT_MESSAGE_CONFIG_SERVER_ADD "c_CONFIGSERVERADD"
/*
 * DIRECTION:
 *	client->manager
 *
 * MESSAGE:
 *	(list)
 *	    [SpecServer]
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *	0 - invalid server specification
 *
 */

#define CLIENT_MESSAGE_CONFIG_SERVER_EDIT "c_CONFIGSERVEREDIT"
/*
 * DIRECTION:
 *	client->manager
 *
 * MESSAGE:
 *	(list)
 *	    name(string)
 *	    spec(list)
 *		[SpecServer]
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *	0 - invalid server specification
 *
 */

#define CLIENT_MESSAGE_CONFIG_SERVER_REMOVE "c_CONFIGSERVERREMOVE"
/*
 * DIRECTION:
 *	client->manager
 *
 * MESSAGE:
 *	name(string)
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *	0 - invalid server name
 *
 */

#define CLIENT_MESSAGE_CONFIG_SERVER_ADDED "c_CONFIGSERVERADDED"
/*
 * DIRECTION:
 *	manager->client
 *
 * MESSAGE:
 *	server(string)
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *
 */

#define CLIENT_MESSAGE_CONFIG_SERVER_REMOVED "c_CONFIGSERVERREMOVED"
/*
 * DIRECTION:
 *	manager->client
 *
 * MESSAGE:
 *	server(string)
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *
 */

#define CLIENT_MESSAGE_CONFIG_SERVER_RENAMED "c_CONFIGSERVERRENAMED"
/*
 * DIRECTION:
 *	manager->client
 *
 * MESSAGE:
 *	(list)
 *	    server(string)
 *	    newname(string)
 * 
 * RESPONSE:
 *	(none)
 *
 * ERROR:
 *
 */

#define CLIENT_MESSAGE_CONSOLE_GET_BROKEN "c_GETBROKEN"
/*
 * DIRECTION:
 *	client->manager
 *
 * MESSAGE:
 *	console(string)
 *
 * RESPONSE:
 *	broken(string)
 *
 * ERROR:
 *
 */


#endif
