#include <string.h>
#include <time.h>
#include "date.h"

extern Date
date_new(guint year,
	guint month,
	guint day,
	guint hour,
	guint minute,
	guint second) {
    struct tm *tim = g_new0(struct tm, 1);
    time_t t;

    g_assert(year > 1899);
    g_assert(month < 12);
    g_assert(day > 0);
    g_assert(day < 32);
    g_assert(hour < 24);
    g_assert(minute < 60);
    g_assert(second < 60);
    tim->tm_year = year - 1900;
    tim->tm_mon = month;
    tim->tm_mday = day;
    tim->tm_hour = hour;
    tim->tm_min = minute;
    tim->tm_sec = second;
    tim->tm_isdst = -1;
    t = mktime(tim);
    g_free(tim);
    return t;
}

extern Date
date_now(void) {
    return time(NULL);
}

extern Date
date_read(const gchar *str,
	guint length,
	guint *bytes_read) {
    guint year = 0;
    guint month = 0;
    guint day = 0;
    guint hour = 0;
    guint minute = 0;
    guint second = 0;
    guint *x;
    gchar *format = "yyyy-mm-dd hh:MM:ss";
    static guint format_length = 0;
    gint i;
    gint format_error = 0;
    
    if (format_length == 0) {
	format_length = strlen(format);
    }
    if (length < format_length) {
	if (bytes_read != NULL) {
	    *bytes_read = 0;
	}
	return 0;
    }
    for (i = 0; format[i] && format_error == 0; i++) {
	x = NULL;
	switch (format[i]) {
	    case 'y':
		if (x == NULL) {
		    x = &year;
		}
	    case 'm':
		if (x == NULL) {
		    x = &month;
		}
	    case 'd':
		if (x == NULL) {
		    x = &day;
		}
	    case 'h':
		if (x == NULL) {
		    x = &hour;
		}
	    case 'M':
		if (x == NULL) {
		    x = &minute;
		}
	    case 's':
		if (x == NULL) {
		    x = &second;
		}
	}
	if (x) {
	    switch (str[i]) {
		case '0':
		    *x = *x * 10;
		    break;
		case '1':
		    *x = (*x * 10) + 1;
		    break;
		case '2':
		    *x = (*x * 10) + 2;
		    break;
		case '3':
		    *x = (*x * 10) + 3;
		    break;
		case '4':
		    *x = (*x * 10) + 4;
		    break;
		case '5':
		    *x = (*x * 10) + 5;
		    break;
		case '6':
		    *x = (*x * 10) + 6;
		    break;
		case '7':
		    *x = (*x * 10) + 7;
		    break;
		case '8':
		    *x = (*x * 10) + 8;
		    break;
		case '9':
		    *x = (*x * 10) + 9;
		    break;
		default:
		    format_error = i + 1;
	    }
	} else {
	    if (str[i] != format[i]) {
		format_error = i + 1;
	    }
	}
    }
    month -= 1;
    if (year < 1900 || month > 11 || hour > 23 || minute > 59 || second > 59) {
	format_error = i + 1;
    }
    if (format_error == 0) {
	if (bytes_read != NULL) {
	    *bytes_read = format_length;
	}
	return date_new(year, month, day, hour, minute, second);
    } else {
	if (bytes_read != NULL) {
	    *bytes_read = 0;
	}
	return 0;
    }
}

extern gchar *
date_print(Date d) {
    time_t t = d;
    struct tm *tim = localtime(&t); /* FIXME: not thread safe! */
    gchar *result = g_new0(gchar, 20);

    strftime(result, 20, "%Y-%m-%d %H:%M:%S", tim);
    return result;
}

extern void
date_explode(Date d,
	guint *year,
	guint *month,
	guint *day,
	guint *hour,
	guint *minute,
	guint *second) {
    time_t t = d;
    struct tm *tim = localtime(&t); /* FIXME: not thread safe! */

    if (year != NULL) {
	*year = tim->tm_year + 1900;
    }
    if (month != NULL) {
	*month = tim->tm_mon;
    }
    if (day != NULL) {
	*day = tim->tm_mday;
    }
    if (hour != NULL) {
	*hour = tim->tm_hour;
    }
    if (minute != NULL) {
	*minute = tim->tm_min;
    }
    if (second != NULL) {
	*second = tim->tm_sec;
    }
}
