#ifndef __IPMI_H
#define __IPMI_H

#include "message.h"

/*
 * IPMIError - error occurring when attempting to open an IPMI enabled terminal.
 *
 */
typedef enum {
    IPMI_ERROR_NOT_SUPPORTED,
    IPMI_ERROR_IPMICONSOLE
} IPMIError;
#define IMPI_ERROR ipmi_error_quark()
extern GQuark ipmi_error_quark(void);


/*
 * IPMI - a struct representing a console on an IPMI enabled host
 *
 * This should be considered read-only.
 *
 */
typedef struct _IPMI {
    gchar *host;
    gchar *user;
    gchar *passwd;
} IPMI;

/*
 * impi_new - create an ipmi structure
 *
 * The returned IPMI belongs to the caller.
 *
 */
extern IPMI *ipmi_new(const gchar *host,
	const gchar *user,
	const gchar *passwd);

/*
 * ipmi_message_arg - generate a message arg that encapsulates the IPMI
 *
 * The returned MessageArg belongs to the caller.
 *
 */
extern MessageArg *ipmi_message_arg(const gchar *name,
	const IPMI *ipmi);

/*
 * ipmi_new_from_arg - create an IPMI from a MessageArg
 *
 * The argument should have been created with ipmi_message_arg().
 * Returns the IPMI, or NULL if the MessageArg was invalid.
 * The returned IPMI belongs to the caller.
 *
 */
extern IPMI *ipmi_new_from_arg(const MessageArg *arg);

/*
 * ipmi_match - compare to IPMIs
 *
 * Returns TRUE id they match.
 *
 */
extern gboolean ipmi_match(const IPMI *a, const IPMI *b);

/*
 * ipmi_free - frees an IPMI structure and all its resources
 *
 */
extern void ipmi_free(IPMI *ipmi);

/*
 * ipmi_open_io_channel - uses an IMPI structure tomake a connection to
 * the hosts console.
 *
 * Returns a GIOChannel connected to the console or fails, returning
 * NULL and setting an error.
 *
 */
extern GIOChannel *ipmi_open_io_channel(const IPMI *ipmi, GError **error);

#endif
