#include "log-query.h"

extern void
log_query_free(LogQuery *lq) {
    g_assert(lq);
    g_assert(lq->log);
    g_free(lq->log);
    if (lq->console != NULL) {
	g_free(lq->console);
    }
    g_free(lq);
}

extern LogQuery *
log_query_new(const gchar *log,
	guint64 start_time,
	guint64 end_time,
	guint max_lines,
	const gchar *console) {
    LogQuery *lq = g_new0(struct _LogQuery, 1);

    g_assert(log);
    lq->log = g_strdup(log);
    lq->start_time = start_time;
    lq->end_time = end_time;
    lq->max_lines = max_lines;
    if (console != NULL) {
	lq->console = g_strdup(console);
    }
    return lq;
}

extern MessageArg *
log_query_message_arg(const gchar *name,
	const LogQuery *lq) {
    GList *l = NULL;

    g_assert(lq);
    g_assert(lq->log);
    l = g_list_append(l, message_arg_string("log", lq->log));
    l = g_list_append(l, message_arg_uint64("start_time", lq->start_time));
    l = g_list_append(l, message_arg_uint64("end_time", lq->end_time));
    l = g_list_append(l, message_arg_uint32("max_lines", lq->max_lines));
    if (lq->console != NULL) {
	l = g_list_append(l, message_arg_string("console", lq->console));
    }
    return message_arg_list(name, l);
}

extern LogQuery *
log_query_new_from_arg(const MessageArg *arg) {
    MessageArg *log, *start_time, *end_time, *max_lines;
    MessageArg *console = NULL;

    g_assert(arg);
    if (arg->type != MESSAGE_ARG_LIST) return NULL;
    if (message_arg_list_explode(arg,
		"log", MESSAGE_ARG_STRING, &log,
		"start_time", MESSAGE_ARG_UINT64, &start_time,
		"end_time", MESSAGE_ARG_UINT64, &end_time,
		"max_lines", MESSAGE_ARG_UINT32, &max_lines,
		NULL, 0, NULL) == FALSE) {
	return NULL;
    }
    message_arg_list_explode(arg,
	    "console", MESSAGE_ARG_STRING, &console,
	    NULL, 0, NULL);
    return log_query_new(log->string->str, start_time->uint64,
	    end_time->uint64, max_lines->uint32,
	    console?console->string->str:NULL);
}
