#ifndef __LOG_QUERY_H
#define __LOG_QUERY_H

#include "message.h"

/*
 * LogQuery - a structure representing a query on a log file
 *
 */
typedef struct _LogQuery {
    gchar *log;
    guint64 start_time;
    guint64 end_time;
    guint max_lines; /* 0 means unlimited */
    gchar *console; /* NULL means any console */
} LogQuery;

/*
 * log_query_new - create a LogQuery structure.
 *
 * The returned LogQuery belongs to the caller.
 *
 */
extern LogQuery * log_query_new(const gchar *log,
	guint64 start_time,
	guint64 end_time,
	guint max_lines,
	const gchar *console);

/*
 * log_query_message_arg - generate a message arg that encapsultes the LogQuery.
 *
 * The returned MessageArg belongs to the caller.
 *
 */
extern MessageArg *log_query_message_arg(const gchar *name,
	const LogQuery *lq);

/*
 * log_query_new_from_arg - generate a LogQuery struct from a MessageArg
 *
 * The argument should have been created with log_query_message_arg().
 * Returns the LogQuery, or NULL if the MessageArg was invalid.
 * The returned LogQuery belongs to the caller.
 *
 */
extern LogQuery *log_query_new_from_arg(const MessageArg *arg);

/*
 * log_query_free - frees a LogQuery structure and all its resources
 *
 */
extern void log_query_free(LogQuery *lq);


#endif
