#ifndef __MATRIX_H
#define __MATRIX_H

#include "glib.h"

/*
 * Matrix - opaque structure representing a IO matrix
 *
 */
typedef struct _Matrix *Matrix;

/*
 * matrix_new - create a new IO matrix
 *
 */
extern Matrix matrix_new(void);

/*
 * matrix_connect - connect two IO channels
 *
 * Returns TRUE or fails, setting an error.
 * After a successful connection any data on "from" will be sent to "to".
 *
 */
extern gboolean matrix_connect(Matrix matrix,
	GIOChannel *from,
	GIOChannel *to,
	GError **error);

/*
 * matrix_disconnect - disconnect two connected channels
 *
 * Supplied the same arguments used to connect the channels, the connection
 * will be broken.
 * Returns TRUE or fails, setting an error.
 *
 */
extern gboolean matrix_disconnect(Matrix matrix,
	GIOChannel *from,
	GIOChannel *to,
	GError **error);

/*
 * matrix_expunge - remove all references to an IO channel
 *
 * Remove any refernces to the given IO channel whether it be the
 * source or destination in the IO matrix.
 *
 */
extern void matrix_expunge(Matrix matrix, GIOChannel *chan);

#endif
