#include "retry.h"

struct _Retry {
    RetryCB retry_cb;
    RetryDoneCB retry_done_cb;
    gdouble multiplier;
    gint max_retries;
    guint interval;
    guint attempt_no;
    guint timeout;
    gpointer user_data;
};

static gboolean
retry_attempt(gpointer data) {
    Retry r = data;

    r->attempt_no += 1;
    if (r->retry_cb(r, r->attempt_no, r->user_data) == TRUE) {
	r->timeout = 0;
	r->retry_done_cb(r, TRUE, r->attempt_no, r->user_data);
	return FALSE;
    }
    if (r->attempt_no == r->max_retries) {
	r->timeout = 0;
	r->retry_done_cb(r, FALSE, r->attempt_no, r->user_data);
	return FALSE;
    }
    r->interval = (guint)(r->interval * r->multiplier);
    r->timeout = g_timeout_add(r->interval, retry_attempt, r);
    return FALSE;
}

extern Retry
retry_new(RetryCB retry_cb,
	RetryDoneCB retry_done_cb,
	gint initial_frequency,
	gdouble multiplier,
	gint max_retries,
	gpointer user_data) {
    Retry r = g_new0(struct _Retry, 1);

    g_assert(retry_cb);
    g_assert(retry_done_cb);

    r->retry_cb = retry_cb;
    r->retry_done_cb = retry_done_cb;
    r->multiplier = multiplier;
    r->max_retries = max_retries;
    r->user_data = user_data;
    r->interval = initial_frequency * 1000;
    r->attempt_no = 0;
    r->timeout = g_timeout_add(r->interval, retry_attempt, r);
    return r;
}

extern void
retry_free(Retry retry) {
    if (retry->timeout != 0) {
	g_source_remove(retry->timeout);
    }
    g_free(retry);
}
