#ifndef __RETRY_H
#define __RETRY_H

#include <glib.h>

/*
 * Retry - an opaque structure representing an event that is repeated
 * after set intervals until a condition is met.
 *
 */
typedef struct _Retry *Retry;

/*
 * RetryCB - called when a retry should be attempted.
 *
 * This callback should return TRUE id the retry was successful.
 */
typedef gboolean (*RetryCB) (Retry retry,
	gint attempt_no,
	gpointer user_data);

/*
 * RetryDoneCB - called when retrying has finished
 *
 */
typedef void (*RetryDoneCB) (Retry retry,
	gboolean successful,
	gint attempt_no,
	gpointer user_data);

/*
 * retry_new
 *
 * The retry_cb will be called after initial_frequency seconds, and if
 * the retry fails, again after initial_frequency*multiplier and again
 * until either the retry succeds, or max_retries is reached. In either
 * case the retry_done_cb will be called.
 * 
 * max_retries may be less than 1, in which case retrying will continue
 * indefinitely.
 *
 */
extern Retry retry_new(RetryCB retry_cb,
	RetryDoneCB retry_done_cb,
	gint initial_frequency,
	gdouble multiplier,
	gint max_retries,
	gpointer user_data);


/*
 * retry_free - free a Retry and all its resources.
 *
 * No callbacks will be issues during this function.
 *
 */
extern void retry_free(Retry retry);

#endif
