#include <string.h>
#include "serialize.h"

static gboolean
deserialize_uint(GString **string,
	guint64 *data,
	gint length) {
    gint i;
    guint64 res = 0;

    g_assert(length == 1 || length == 2 || length == 4 || length == 8);
    if ((*string)->len < length) return FALSE;
    for (i = 0; i < length; i++) {
        res = (res * 256) + (guint8) ((*string)->str[0]);
        *string = g_string_erase(*string, 0, 1);
    }
    *data = res;
    return TRUE;
}

static GString *
serialize_uint(guint64 data,
	gint length) {
    gint i;
    gchar buf[8];

    g_assert(length == 1 || length == 2 || length == 4 || length == 8);
    for (i = 0; i < length; i++) {
        buf[7 - i] = data % 256;
        data = data >> 8;
    }
    return g_string_new_len(&buf[8 - length], length);
}

gboolean
deserialize_uint8(GString **string,
	guint8 *data) {
    gboolean res;
    guint64 ret;

    res = deserialize_uint(string, &ret, 1);
    if (res) {
        *data = (guint8) ret;
    }
    return res;
}

GString *
serialize_uint8(guint8 data) {
    return serialize_uint((guint64) data, 1);
}


gboolean
deserialize_uint16(GString **string,
	guint16 *data) {
    gboolean res;
    guint64 ret;

    res = deserialize_uint(string, &ret, 2);
    if (res) {
        *data = (guint16) ret;
    }
    return res;
}

GString *
serialize_uint16(guint16 data) {
    return serialize_uint((guint64) data, 2);
}

gboolean
deserialize_uint32(GString **string,
	guint32 *data) {
    gboolean res;
    guint64 ret;

    res = deserialize_uint(string, &ret, 4);
    if (res) {
        *data = (guint32) ret;
    }
    return res;
}

GString *
serialize_uint32(guint32 data) {
    return serialize_uint((guint64) data, 4);
}


gboolean
deserialize_uint64(GString **string,
	guint64 *data) {
    gboolean res;
    guint64 ret;

    res = deserialize_uint(string, &ret, 8);
    if (res) {
        *data = ret;
    }
    return res;
}

GString *
serialize_uint64(guint64 data) {
    return serialize_uint(data, 8);
}

gboolean
deserialize_string(GString **string,
	GString **data) {
    GString *number;
    guint32 string_length;

    if ((*string)->len < 4) return FALSE;
    number = g_string_new_len((*string)->str, 4);
    deserialize_uint32(&number, &string_length);
    g_string_free(number, TRUE);
    if ((*string)->len < (4 + string_length)) return FALSE;
    *string = g_string_erase(*string, 0, 4);
    *data = g_string_new_len((*string)->str, string_length);
    *string = g_string_erase(*string, 0, string_length);
    return TRUE;
}

GString *
serialize_string(GString *data) {
    GString *res = serialize_uint32(data->len);

    res = g_string_append_len(res, data->str, data->len);
    return res;
}

GString *
serialize_string_gchar(const gchar *data) {
    GString *res = serialize_uint32(strlen(data));

    res = g_string_append(res, data);
    return res;
}
