#include <string.h>
#include "spec-cgroup.h"

#define SPEC_CGROUP_NAME "SpecCgroup_name"
#define SPEC_CGROUP_CONSOLES "SpecCgroup_consoles"

extern void
spec_cgroup_free(SpecCgroup *sg) {
    g_assert(sg);
    if (sg->name) {
	g_free(sg->name);
    }
    while (sg->consoles) {
	if (sg->consoles->data) {
	    g_free(sg->consoles->data);
	}
	sg->consoles = g_list_remove(sg->consoles, sg->consoles->data);
    }
    g_free(sg);
}

extern MessageArg *
spec_cgroup_message_arg(const gchar *name,
	const SpecCgroup *sg) {
    MessageArg *a;
    MessageArg *consoles;
    GList *l;

    g_assert(name);
    g_assert(sg);

    a = message_arg_list(name, NULL);
    a->list = g_list_prepend(a->list,
	    message_arg_string(SPEC_CGROUP_NAME, sg->name?sg->name:""));
    consoles = message_arg_list(SPEC_CGROUP_CONSOLES, NULL);
    a->list = g_list_prepend(a->list, consoles);
    for (l = sg->consoles; l != NULL; l = g_list_next(l)) {
	consoles->list = g_list_prepend(consoles->list,
		message_arg_string("", l->data));
    }
    return a;
}

extern SpecCgroup *
spec_cgroup_new_from_arg(const MessageArg *arg) {
    MessageArg *name;
    MessageArg *consoles;
    SpecCgroup *sg;
    GList *l;

    g_assert(arg);

    if (arg->type != MESSAGE_ARG_LIST) return NULL;
    if (message_arg_list_explode(arg,
		SPEC_CGROUP_NAME, MESSAGE_ARG_STRING, &name,
		SPEC_CGROUP_CONSOLES, MESSAGE_ARG_LIST, &consoles,
		NULL, 0, NULL) == FALSE) {
	return NULL;
    }
    sg = g_new0(struct _SpecCgroup, 1);
    sg->name = g_strdup(name->string->str);
    sg->consoles = NULL;
    for (l = consoles->list; l != NULL; l = g_list_next(l)) {
	MessageArg *a = l->data;

	if (a->type != MESSAGE_ARG_STRING) {
	    continue;
	}
	sg->consoles = g_list_prepend(sg->consoles,
		g_strdup(a->string->str));
    }
    return sg;
}
