#ifndef __SPEC_CONSOLE_H
#define __SPEC_CONSOLE_H

#include "message.h"
#include "terminal.h"
#include "ipmi.h"

/*
 * ConsoleType - connected serial device or IPMI connection
 *
 */
typedef enum {
    CONSOLE_TYPE_SERIAL,
    CONSOLE_TYPE_IPMI
} ConsoleType;

/*
 * SpecConsole - a structure representing a specification of a console
 *
 */
typedef struct _SpecConsole {
    gchar *name;
    gchar *server;
    ConsoleType type;
    union {
	Terminal *terminal;
	IPMI *ipmi;
    };
} SpecConsole;

/*
 * spec_console_free - free a previously allocated SpecConsole
 *
 */
extern void spec_console_free(SpecConsole *sc);

/*
 * spec_console_serial_new - allocate a new serial-type SpecConsole
 *
 * The bits argument should be either "8N1" or "7E2".
 * The returned SpecConsole belongs to the caller.
 *
 */
extern SpecConsole *spec_console_serial_new(const gchar *name,
	const gchar *server,
	const gchar *device,
	guint speed,
	const gchar *bits);

/*
 * spec_console_ipmi_new - allocate a new IPMI-type SpecConsole
 *
 * The returned SpecConsole belongs to the caller.
 *
 */
extern SpecConsole *spec_console_ipmi_new(const gchar *name,
	const gchar *server,
	const gchar *host,
	const gchar *user,
	const gchar *passwd);


/*
 * spec_console_message_arg - generate a message arg that encapsulates the SpecConsole
 *
 */
extern MessageArg *spec_console_message_arg(const gchar *name,
	const SpecConsole *sc);

/*
 * spec_console_new_from_arg - generate a SpecConsole from a message arg
 *
 * The arg should have been created with spec_console_message_arg().
 * Returns a newly allocated, caller owned SpecConsole, or NULL if arg was
 * invalid.
 *
 */
extern SpecConsole *spec_console_new_from_arg(const MessageArg *arg);

#endif
