#include <string.h>
#include "spec-log.h"

#define SPEC_LOG_NAME		"SpecLog_name"
#define SPEC_LOG_SERVER		"SpecLog_server"
#define SPEC_LOG_FILE		"SpecLog_file"
#define SPEC_LOG_CONSOLES	"SpecLog_consoles"

extern void
spec_log_free(SpecLog *sl) {
    g_assert(sl);
    if (sl->name) {
	g_free(sl->name);
    }
    if (sl->server) {
	g_free(sl->server);
    }
    if (sl->file) {
	g_free(sl->file);
    }
    while (sl->consoles != NULL) {
	if (sl->consoles->data) {
	    g_free(sl->consoles->data);
	}
	sl->consoles = g_list_remove(sl->consoles, sl->consoles->data);
    }
    g_free(sl);
}

extern MessageArg *
spec_log_message_arg(const gchar *name,
	const SpecLog *sl) {
    MessageArg *a;
    MessageArg *consoles;
    GList *l;

    g_assert(name);
    g_assert(sl);

    a = message_arg_list(name, NULL);
    a->list = g_list_prepend(a->list,
	    message_arg_string(SPEC_LOG_NAME, sl->name?sl->name:""));
    a->list = g_list_prepend(a->list,
	    message_arg_string(SPEC_LOG_SERVER, sl->server?sl->server:""));
    a->list = g_list_prepend(a->list,
	    message_arg_string(SPEC_LOG_FILE, sl->file?sl->file:""));
    consoles = message_arg_list(SPEC_LOG_CONSOLES, NULL);
    a->list = g_list_prepend(a->list, consoles);
    for (l = sl->consoles; l != NULL; l = g_list_next(l)) {
	consoles->list = g_list_prepend(consoles->list,
		message_arg_string("", l->data));
    }
    return a;
}

extern SpecLog *
spec_log_new_from_arg(const MessageArg *arg) {
    MessageArg *name;
    MessageArg *server;
    MessageArg *file;
    MessageArg *consoles;
    SpecLog *sl;
    GList *l;

    g_assert(arg);
    if (arg->type != MESSAGE_ARG_LIST) {
	return NULL;
    }
    if (message_arg_list_explode(arg,
		SPEC_LOG_NAME, MESSAGE_ARG_STRING, &name,
		SPEC_LOG_SERVER, MESSAGE_ARG_STRING, &server,
		SPEC_LOG_FILE, MESSAGE_ARG_STRING, &file,
		SPEC_LOG_CONSOLES, MESSAGE_ARG_LIST, &consoles,
		NULL, 0, NULL) == FALSE) {
	return NULL;
    }

    sl = g_new0(struct _SpecLog, 1);
    sl->name = g_strdup(name->string->str);
    sl->server = g_strdup(server->string->str);
    sl->file = g_strdup(file->string->str);
    sl->consoles = NULL;
    for (l = consoles->list; l != NULL; l = g_list_next(l)) {
	MessageArg *console = l->data;

	if (console->type == MESSAGE_ARG_STRING) {
	    sl->consoles = g_list_prepend(sl->consoles,
		    g_strdup(console->string->str));
	}
    }
    return sl;
}
