#include <string.h>
#include "spec-server.h"

#define SPEC_SERVER_NAME	    "SpecServer_name"
#define SPEC_SERVER_TYPE	    "SpecServer_type"
#define SPEC_SERVER_PATH	    "SpecServer_path"
#define SPEC_SERVER_HOST	    "SpecServer_host"
#define SPEC_SERVER_PORT	    "SpecServer_port"
#define SPEC_SERVER_SSH_PROGRAM	    "SpecServer_ssh_program"
#define SPEC_SERVER_SSH_USER	    "SpecServer_ssh_user"
#define SPEC_SERVER_SSH_IDENTITY    "SpecServer_ssh_identity"
#define SPEC_SERVER_REMOTE_COMMAND  "SpecServer_remote_command"

extern void
spec_server_free(SpecServer *ss) {
    g_assert(ss);
    if (ss->name) {
	g_free(ss->name);
    }
    switch (ss->type) {
	case SERVER_TYPE_EXEC:
	    if (ss->exec.path) {
		g_free(ss->exec.path);
	    }
	    break;
	case SERVER_TYPE_CONNECT:
	    if (ss->connect.host) {
		g_free(ss->connect.host);
	    }
	    break;
	case SERVER_TYPE_SSH:
	    if (ss->ssh.host) {
		g_free(ss->ssh.host);
	    }
	    if (ss->ssh.ssh_program) {
		g_free(ss->ssh.ssh_program);
	    }
	    if (ss->ssh.ssh_user) {
		g_free(ss->ssh.ssh_user);
	    }
	    if (ss->ssh.ssh_identity) {
		g_free(ss->ssh.ssh_identity);
	    }
	    if (ss->ssh.remote_cmd) {
		g_free(ss->ssh.remote_cmd);
	    }
	    break;
	default:
	    g_assert_not_reached();
    }
    g_free(ss);
}

extern MessageArg *
spec_server_message_arg(const gchar *name,
	const SpecServer *ss) {
    MessageArg *a;

    g_assert(name);
    g_assert(ss);
    a = message_arg_list(name, NULL);
    a->list = g_list_prepend(a->list,
	    message_arg_string(SPEC_SERVER_NAME, ss->name?ss->name:""));
    a->list = g_list_prepend(a->list,
	    message_arg_uint8(SPEC_SERVER_TYPE, ss->type));
    switch (ss->type) {
	case SERVER_TYPE_EXEC:
	    a->list = g_list_prepend(a->list,
		    message_arg_string(SPEC_SERVER_PATH,
			ss->exec.path?ss->exec.path:""));
	    break;
	case SERVER_TYPE_CONNECT:
	    a->list = g_list_prepend(a->list,
		    message_arg_string(SPEC_SERVER_HOST,
			ss->connect.host?ss->connect.host:""));
	    a->list = g_list_prepend(a->list,
		    message_arg_uint16(SPEC_SERVER_PORT, ss->connect.port));
	    break;
	case SERVER_TYPE_SSH:
	    a->list = g_list_prepend(a->list,
		    message_arg_string(SPEC_SERVER_HOST,
			ss->ssh.host?ss->ssh.host:""));
	    a->list = g_list_prepend(a->list,
		    message_arg_string(SPEC_SERVER_SSH_PROGRAM,
			ss->ssh.ssh_program?ss->ssh.ssh_program:""));
	    a->list = g_list_prepend(a->list,
		    message_arg_string(SPEC_SERVER_SSH_USER,
			ss->ssh.ssh_user?ss->ssh.ssh_user:""));
	    a->list = g_list_prepend(a->list,
		    message_arg_string(SPEC_SERVER_SSH_IDENTITY,
			ss->ssh.ssh_identity?ss->ssh.ssh_identity:""));
	    a->list = g_list_prepend(a->list,
		    message_arg_string(SPEC_SERVER_REMOTE_COMMAND,
			ss->ssh.remote_cmd?ss->ssh.remote_cmd:""));
	    break;
	default:
	    g_assert_not_reached();
    }
    return a;
}

extern SpecServer *
spec_server_new_from_arg(const MessageArg *arg) {
    MessageArg *name;
    MessageArg *type;
    MessageArg *path;
    MessageArg *host;
    MessageArg *port;
    MessageArg *ssh_program;
    MessageArg *ssh_user;
    MessageArg *ssh_identity;
    MessageArg *remote_cmd;
    SpecServer *ss;

    g_assert(arg);
    if (arg->type != MESSAGE_ARG_LIST) return NULL;
    if (message_arg_list_explode(arg,
		SPEC_SERVER_NAME, MESSAGE_ARG_STRING, &name,
		SPEC_SERVER_TYPE, MESSAGE_ARG_UINT8, &type,
		NULL, 0, NULL) == FALSE) {
	return NULL;
    }
    switch (type->uint8)  {
	case SERVER_TYPE_EXEC:
	    if (message_arg_list_explode(arg,
			SPEC_SERVER_PATH, MESSAGE_ARG_STRING, &path,
			NULL, 0, NULL) == FALSE) {
		return NULL;
	    }
	    break;
	case SERVER_TYPE_CONNECT:
	    if (message_arg_list_explode(arg,
			SPEC_SERVER_HOST, MESSAGE_ARG_STRING, &host,
			SPEC_SERVER_PORT, MESSAGE_ARG_UINT16, &port,
			NULL, 0, NULL) == FALSE) {
		return NULL;
	    }
	    break;
	case SERVER_TYPE_SSH:
	    if (message_arg_list_explode(arg,
			SPEC_SERVER_HOST, MESSAGE_ARG_STRING, &host,
			SPEC_SERVER_SSH_PROGRAM, MESSAGE_ARG_STRING, &ssh_program,
			SPEC_SERVER_SSH_USER, MESSAGE_ARG_STRING, &ssh_user,
			SPEC_SERVER_SSH_IDENTITY, MESSAGE_ARG_STRING, &ssh_identity,
			SPEC_SERVER_REMOTE_COMMAND, MESSAGE_ARG_STRING, &remote_cmd,
			NULL, 0, NULL) == FALSE) {
		return NULL;
	    }
	    break;
	default:
	    return NULL;
    }
    ss = g_new0(struct _SpecServer, 1);
    ss->name = g_strdup(name->string->str);
    ss->type = type->uint8;
    switch (ss->type) {
	case SERVER_TYPE_EXEC:
	    ss->exec.path = g_strdup(path->string->str);
	    break;
	case SERVER_TYPE_CONNECT:
	    ss->connect.host = g_strdup(host->string->str);
	    ss->connect.port = port->uint16;
	    break;
	case SERVER_TYPE_SSH:
	    ss->ssh.host = g_strdup(host->string->str);
	    ss->ssh.ssh_program = g_strdup(ssh_program->string->str);
	    ss->ssh.ssh_user = g_strdup(ssh_user->string->str);
	    ss->ssh.ssh_identity = g_strdup(ssh_identity->string->str);
	    ss->ssh.remote_cmd = g_strdup(remote_cmd->string->str);
	    break;
	default:
	    g_assert_not_reached();
    }
    return ss;
}
