#ifndef __SPEC_SERVER_H
#define __SPEC_SERVER_H

#include "message.h"

/*
 * ServerType - connection type of a server
 *
 */
typedef enum {
    SERVER_TYPE_EXEC,
    SERVER_TYPE_CONNECT,
    SERVER_TYPE_SSH
} ServerType;

/*
 * ServerExec - a structure representing an executed server
 *
 */
typedef struct _ServerExec {
    gchar *path;
} ServerExec;

/*
 * ServerConnect -  a structure representing a TCP connected server
 *
 */
typedef struct _ServerConnect {
    gchar *host;
    gint port;
} ServerConnect;

/*
 * ServerSsh - a streucture representing an SSH connected server
 *
 */
typedef struct _ServerSsh {
    gchar *host;
    gchar *ssh_program;
    gchar *ssh_user;
    gchar *ssh_identity;
    gchar *remote_cmd;
} ServerSsh;

/*
 * SpecServer - a structure representing a specification of a server
 *
 */
typedef struct _SpecServer {
    gchar *name;
    ServerType type;
    union {
	ServerExec exec;
	ServerConnect connect;
	ServerSsh ssh;
    };
} SpecServer;

/*
 * spec_server_free - free a previously allocated SpecServer
 *
 */
extern void spec_server_free(SpecServer *ss);

/*
 * spec_server_message_arg - generate a message arg encapsulating the SpecServer
 *
 */
extern MessageArg *spec_server_message_arg(const gchar *name,
	const SpecServer *ss);

/*
 * spec_server_new_from_arg - generate a SpecServer from a message arg
 *
 * The arg should have been created with spec_message_arg().
 * Returns a newly allocated, caller owned SpecServer, or NULL if arg was
 * invalid.
 *
 */
extern SpecServer *spec_server_new_from_arg(const MessageArg *arg);


#endif
