#ifndef __CGROUP_H
#define __CGROUP_H

#include "mgr.h"
#include "status.h"
#include "spec-cgroup.h"

/*
 * CGroup - opaque structure representing a console
 *
 */
typedef struct _CGroup *CGroup;

/*
 * cgroup_load - load console group configuration data
 *
 */
extern void cgroup_load(Manager *manager);

/*
 * cgroup_list - return a list of all console groups
 *
 * A list of console names is returned. The list and its contents
 * belong to the caller
 *
 */
extern GList *cgroup_list(Manager *manager);

/*
 * cgroup_list_consoles - return a list of consoles in a group
 *
 * A list of console names is returned. The list and its contents
 * belong to the caller
 *
 */
extern GList *cgroup_list_consoles(CGroup g);

/*
 * cgroup_get_status
 *
 * Returns the current status of the console group
 *
 */
extern Status cgroup_get_status(CGroup g);

/*
 * cgroup_check_status - calculate the statuses of all groups.
 *
 */
extern void cgroup_check_status(Manager *manager);

/*
 * cgroup_spec - return a specification of a console group
 *
 */
extern SpecCgroup *cgroup_spec(Manager *manager,
	const gchar *cgroup,
	GError **error);

/*
 * cgroup_add - add a new cgroup
 *
 * Returns TRUE if the console is created or FALSE and an error
 *
 */
extern gboolean cgroup_add(Manager *manager,
	const SpecCgroup *spec,
	GError **error);

/*
 * cgroup_edit - edit an existing cgroup
 *
 * Returns TRUE if the edit was successful or FALSE and an error
 *
 */
extern gboolean cgroup_edit(Manager *manager,
	const gchar *cgroup,
	const SpecCgroup *spec,
	GError **error);

/*
 * cgroup_remove - remove an existing cgroup
 *
 * Returns TRUE if the group is removed or FALSE and an error
 *
 */
extern gboolean cgroup_remove(Manager *manager,
	const gchar *cgroup,
	GError **error);

#endif
