#ifndef __CONSOLE_H
#define __CONSOLE_H

#include "mgr.h"
#include "status.h"
#include "spec-console.h"

/*
 * Console - opaque structure representing a console
 *
 */
typedef struct _Console *Console;

/*
 * console_load - load console configuration data
 *
 */
extern void console_load(Manager *manager);

/*
 * console_get_io_channel
 *
 * Returns the GIOChannel associated with the console, or NULL if
 * its GIOChannel does not exist.
 *
 */
extern GIOChannel *console_get_io_channel(Console c);

/*
 * console_get_status
 *
 * Returns the current status of the given console.
 *
 */
extern Status console_get_status(Console c);

/*
 * console_get_broken - returns a reason for a console not working
 *
 * Will return NULL if the console is working, or if there is now
 * known reason for the breakage.
 *
 */
extern G_CONST_RETURN gchar *console_get_broken(Console c);

/*
 * console_get_last_active
 *
 * Returns the current status of the given console.
 *
 */
extern guint64 console_get_last_active(Console c);

/*
 * console_spec - return a specification of a console
 *
 */
extern SpecConsole *console_spec(Manager *manager,
	const gchar *console,
	GError **error);

/*
 * console_list - return a list of all consoles
 *
 * A list of console names is returned. The list and its contents
 * belong to the caller
 *
 */
extern GList *console_list(Manager *manager);

/*
 * console_list_per_server - return a list of all consoles attached to a server
 *
 * A list of console names is returned. The list and its contents
 * belong to the caller
 *
 */
extern GList *console_list_per_server(Manager *manager,
	const gchar *server);

/*
 * console_add - add a new console
 *
 * Returns TRUE is the console sucessfully added or FALSE, and an error
 *
 */
extern gboolean console_add(Manager *manager,
	const SpecConsole *spec,
	GError **error);

/*
 * console_edit - edit an existing console
 *
 * Returns TRUE is the console sucessfully editted or FALSE, and an error
 *
 */
extern gboolean console_edit(Manager *manager,
	const gchar *console,
	const SpecConsole *spec,
	GError **error);

/*
 * console_remove - remove an existing console
 *
 * Returns TRUE is console sucessfully removed, or FALSE and an errror
 *
 */
extern gboolean console_remove(Manager *manager,
	const gchar *console,
	GError **error);

#endif
