#ifndef __LOG_H
#define __LOG_H

#include "mgr.h"
#include "log-query.h"
#include "spec-log.h"

/*
 * Log - an opaque structure representing a log
 *
 */
typedef struct _Log *Log;

/*
 * LogQueryCB - callback initiated when a query has initiated
 * - see log_query() . data_channel can be read to get results
 * of the query.
 *
 */
typedef void (*LogQueryCB) (Log l,
	GIOChannel *data_channel,
	gpointer data);

/*
 * log_load - load log configuration data
 *
 */
extern void log_load(Manager *manager);


/*
 * log_query - start a log inspection
 *
 * Given a set of query parameters, initiate a log inspection.
 * Callback is called when the search data is available.
 *
 */
extern void log_query(Log l,
	const LogQuery *lq,
	LogQueryCB cb,
	gpointer data);

/*
 * log_spec - return a specification of a log
 *
 */
extern SpecLog *log_spec(Manager *manager,
	const gchar *log,
	GError **error);

/*
 * log_list - return a list of all logs
 *
 * A list of log names is returned. This list and its contents belong to
 * the caller
 *
 */
extern GList *log_list(Manager *manager);

/*
 * log_list_per_server - return a list of all logs attached to a server
 *
 * A list of log names is returned. The list and its contents
 * belong to the caller
 *
 */
extern GList *log_list_per_server(Manager *manager,
	const gchar *server);

/*
 * log_list_per_console - return a list of all logs that log a given console
 *
 * A list of log names is returned. The list and its contents
 * belong to the caller
 *
 */
extern GList *log_list_per_console(Manager *manager,
	const gchar *console);


/*
 * log_add - add a new log
 *
 * Returns TRUE is the log sucessfully added or FALSE, and an error
 *
 */
extern gboolean log_add(Manager *manager,
	const SpecLog *spec,
	GError **error);

/*
 * log_edit - edit an existing log
 *
 * Returns TRUE is the log sucessfully editted or FALSE, and an error
 *
 */
extern gboolean log_edit(Manager *manager,
	const gchar *log,
	const SpecLog *spec,
	GError **error);

/*
 * log_remove - remove an existing log
 *
 * Returns TRUE is log sucessfully removed, or FALSE and an errror
 *
 */
extern gboolean log_remove(Manager *manager,
	const gchar *log,
	GError **error);

#endif
