#ifndef __SERVER_CONNECTION_H
#define __SERVER_CONNECTION_H

#include <glib.h>
#include "message.h"
#include "mplex.h"

/*
 * ServerConnecitonError - error generated whilst connecting to servers
 *
 */
typedef enum {
    SERVER_CONNECTION_FAILED
} ServerConnectionError;
#define SERVER_CONNECTION_ERROR server_connection_error_quark()
extern GQuark server_connection_error_quark(void);

/*
 * ServerConnection - opaque structure representing a connetion to a server.
 *
 */
typedef struct _ServerConnection *ServerConnection;

/*
 * ServerConnectionClosedCB - called when a connection closes.
 *
 * The error may be NULL, if no error was available.
 * If set, it belongs to the callee.
 *
 */
typedef void (*ServerConnectionClosedCB) (ServerConnection s,
	gpointer user_data,
	GError *err);

/*
 * server_exec - execute a concsvr on the local machine
 *
 * The path is optional (it may be null).
 * Returns the newly created connection, or fails returning NULL and an error.
 *
 */
extern ServerConnection server_exec(const gchar *path,
	ServerConnectionClosedCB cb,
	gpointer user_data,
	GError **error);

/*
 * server_connect - connect to a server
 *
 * The port may be negative, in which case the default port number will
 * be used.
 * Returns the newly created connection, or fails returning NULL and an error.
 *
 */
extern ServerConnection server_connect(const gchar *host,
	gint port,
	ServerConnectionClosedCB cb,
	gpointer user_data,
	GError **error);

/*
 * server_ssh - use ssh to exec ute a concsvr on  remote machine
 *
 * The ssh executable is specified using the program argument (rsh may be
 * specified, but htis will not work if an identity file has also been
 * specified). The user and identify file specify the remote user name
 * and ssh key to be used, and may be NULL (in which case the default
 * identity will be used and the user will be tha same as the current uid).
 *
 * Returns the newly created connection, or fails returning NULL and an error.
 *
 */
extern ServerConnection server_ssh(const gchar *host,
	const gchar *program,
	const gchar *user,
	const gchar *identity_file,
	const gchar *remote_cmd,
	ServerConnectionClosedCB cb,
	gpointer user_data,
	GError **error);

/*
 * server_connection_free - free a connection and all its resources
 *
 */
extern void server_connection_free(ServerConnection s);

/*
 * server_connection_message_queue
 *
 * Return the message queue of the given server connection.
 *
 */
extern MessageQueue server_connection_message_queue(ServerConnection s);

/*
 * server_connection_mplex
 *
 * Return the message queue of the given server connection.
 *
 */
extern Mplex server_connection_mplex(ServerConnection s);

#endif
