#ifndef __SERVER_H
#define __SERVER_H

#include "mgr.h"
#include "mplex.h"
#include "message.h"
#include "spec-server.h"

/*
 * Server - opaque structre representing a server
 *
 */
typedef struct _Server *Server;

/*
 * server_make_connection - attempt o make a connection to a server.
 *
 * Returns TRUE if the connection has already been make, or was
 * successfully made or FALSE, setting an error.
 *
 */
extern gboolean server_make_connection(Server s,
	GError **error);

/*
 * server_load - load server configuration data
 *
 */
extern void server_load(Manager *manager);

/*
 * server_message_queue - return the message queue for a server
 *
 * If the server message queue is not currently available, returns NULL
 *
 */
extern MessageQueue server_message_queue(Manager *manager,
	const gchar *server);

/*
 * server_mplex - return the server's mplex.
 *
 * If the server;s mplex is not available, returns NULL
 *
 */
extern Mplex server_mplex(Manager *manager,
	const gchar *server);

/*
 * server_list - return a list of all servers
 *
 * A list of server names is returned. The list and its contents
 * belong to the caller
 *
 */
extern GList *server_list(Manager *manager);

/*
 * server_spec - return a specification of a server
 *
 */
extern SpecServer *server_spec(Manager *manager,
	const gchar *server,
	GError **error);

/*
 * server_add - add a new server
 *
 * Returns TRUE if the server is created or FALSE and an error
 *
 */
extern gboolean server_add(Manager *manager,
	const SpecServer *spec,
	GError **error);

/*
 * server_edit - edit an existing server
 *
 * Returns TRUE if the edit was successful or FALSE and an error
 *
 */
extern gboolean server_edit(Manager *manager,
	const gchar *server,
	const SpecServer *spec,
	GError **error);

/*
 * server_remove - remove an existing server
 *
 * Returns TRUE if the server is removed or FALSE and an error
 *
 */
extern gboolean server_remove(Manager *manager,
	const gchar *server,
	GError **error);


#endif
