#ifndef __USER_H
#define __USER_H

#include "mgr.h"
#include "spec-user.h"

/*
 * User - opaque structure representing a user
 *
 */
typedef struct _User *User;

/*
 * user_load - load user_configuration data
 *
 */
extern void user_load(Manager *manager);

/*
 * user_verify - check a user's credentials
 *
 * Returns TRUE if the user and password and valid.
 *
 */
extern gboolean user_verify(Manager *manager,
	const gchar *user,
	const gchar *password);

/*
 * user_change_password - change a user's password
 *
 * Returns TRUE if the password was changed.
 *
 */
extern gboolean user_change_password(Manager *manager,
	const gchar *user,
	const gchar *password);

/*
 * user_add - add a new user
 *
 * Returns TRUE if user sucessfully added or FALSE, and an error
 *
 */
extern gboolean user_add(Manager *manager,
	const SpecUser *spec,
	GError **error);

/*
 * user_edit - edit an existing user
 *
 * Returns TRUE if user editted or FALSE, and an error
 *
 */
extern gboolean user_edit(Manager *manager,
	const gchar *user,
	const SpecUser *spec,
	GError **error);

/*
 * user_remove - remove an existing user
 *
 * Returns TRUE if user successfully removed, or FALSE and an error.
 *
 */
extern gboolean user_remove(Manager *manager,
	const gchar *user,
	GError **error);

/*
 * user_spec - return a specification of a user
 *
 */
extern SpecUser *user_spec(Manager *manager,
	const gchar *user,
	GError **error);

/*
 * user_list - return a list of all users
 *
 * A list of user names is returned. The list and its contents
 * belong to the caller
 *
 */
extern GList *user_list(Manager *manager);

#endif
